# AUTOGENERATED - modify shared_anyscale_util in root directory to make changes
import os
from typing import cast


TEST_MODE = os.environ.get("TEST_MODE") == "1"

ANYSCALE_ENDPOINTS = {
    "development": "https://console.anyscale-dev.dev",
    "staging": "https://console.anyscale-staging.com",
    "production": "https://console.anyscale.com",
    "predeploy": "https://console.predeploy.anyscale.dev",
    "test": "",
}

ANYSCALE_CORS_ORIGINS = {
    # development
    "https://console.anyscale-dev.dev": "https://*.anyscale-dev.dev",
    # staging
    "https://console.anyscale-staging.com": "https://*.anyscale-staging.com",
    # production
    "https://console.anyscale.com": "https://*.anyscale.com",
    # predeploy
    "https://console.predeploy.anyscale.dev": "https://*.predeploy.anyscale.dev",
}

# Required ExposeHeaders for CORS configuration to support file viewer feature
ANYSCALE_CORS_EXPOSE_HEADERS = ["Accept-Ranges", "Content-Range", "Content-Length"]

ANYSCALE_EXTERNAL_API_ENDPOINTS = {
    "development": "https://anyscale-dev.dev/ext",
    "staging": "https://api.anyscale-staging.com",
    "production": "https://api.anyscale.com",
    "predeploy": "https://api.predeploy.anyscale.dev",
    "test": "",
}

if (
    "ANYSCALE_HOST" in os.environ
    and os.environ.get("ANYSCALE_HOST") not in ANYSCALE_ENDPOINTS.values()
):
    anyscale_env_default = "test"
else:
    anyscale_env_default = "production"

ANYSCALE_ENV = cast(
    str,
    os.environ.get("ANYSCALE_DEPLOY_ENVIRONMENT")
    or os.environ.get("DEPLOY_ENVIRONMENT", anyscale_env_default),
)
ANYSCALE_HOST = os.environ.get("ANYSCALE_HOST", ANYSCALE_ENDPOINTS[ANYSCALE_ENV])
ANYSCALE_CORS_ORIGIN = ANYSCALE_CORS_ORIGINS.get(
    ANYSCALE_HOST, "https://*.anyscale-dev.dev"
)
ANYSCALE_API_HOST = ANYSCALE_EXTERNAL_API_ENDPOINTS[ANYSCALE_ENV]



from typing import Optional

import click

import anyscale
from anyscale.aggregated_instance_usage.models import DownloadCSVFilters
from anyscale.cli_logger import BlockLogger
from anyscale.commands import command_examples
from anyscale.commands.util import AnyscaleCommand


log = BlockLogger()  # CLI Logger


@click.group("aggregated-instance-usage", help="Access aggregated instance usage.")
def aggregated_instance_usage_cli() -> None:
    pass


@aggregated_instance_usage_cli.command(
    name="download-csv",
    cls=AnyscaleCommand,
    example=command_examples.AGGREGATED_INSTANCE_USAGE_DOWNLOAD_CSV_EXAMPLE,
)
@click.option(
    "--start-date",
    required=True,
    type=str,
    help="The start date (inclusive) of the aggregated instance usage report. Format: YYYY-MM-DD",
)
@click.option(
    "--end-date",
    required=True,
    type=str,
    help="The end date (inclusive) of the aggregated instance usage report. Format: YYYY-MM-DD",
)
@click.option(
    "--cloud",
    required=False,
    type=str,
    help="The name of the cloud to filter the report by.",
)
@click.option(
    "--project",
    required=False,
    type=str,
    help="The name of the project to filter the report by.",
)
@click.option(
    "--directory",
    required=False,
    type=str,
    help="The directory to save the CSV file to. Default is the current directory.",
)
@click.option(
    "--hide-progress-bar",
    required=False,
    type=bool,
    help="Whether to hide the progress bar. Default is False.",
)
def download_csv(
    start_date: str,
    end_date: str,
    cloud: Optional[str],
    project: Optional[str],
    directory: Optional[str],
    hide_progress_bar: Optional[bool] = False,
) -> None:
    """
    Download an aggregated instance usage report as a zipped CSV to the provided directory.
    """
    log.info("Downloading aggregated instance usage CSV...")
    try:
        anyscale.aggregated_instance_usage.download_csv(
            DownloadCSVFilters(
                start_date=start_date,
                end_date=end_date,
                cloud=cloud,
                project=project,
                directory=directory,
                hide_progress_bar=hide_progress_bar,
            )
        )
    except ValueError as e:
        log.error(f"{e}")
        return



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class UtmFields(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'utm_source': 'str',
        'utm_campaign': 'str',
        'utm_content': 'str',
        'utm_medium': 'str',
        'referrer_url': 'str'
    }

    attribute_map = {
        'utm_source': 'utm_source',
        'utm_campaign': 'utm_campaign',
        'utm_content': 'utm_content',
        'utm_medium': 'utm_medium',
        'referrer_url': 'referrer_url'
    }

    def __init__(self, utm_source=None, utm_campaign=None, utm_content=None, utm_medium=None, referrer_url=None, local_vars_configuration=None):  # noqa: E501
        """UtmFields - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._utm_source = None
        self._utm_campaign = None
        self._utm_content = None
        self._utm_medium = None
        self._referrer_url = None
        self.discriminator = None

        if utm_source is not None:
            self.utm_source = utm_source
        if utm_campaign is not None:
            self.utm_campaign = utm_campaign
        if utm_content is not None:
            self.utm_content = utm_content
        if utm_medium is not None:
            self.utm_medium = utm_medium
        if referrer_url is not None:
            self.referrer_url = referrer_url

    @property
    def utm_source(self):
        """Gets the utm_source of this UtmFields.  # noqa: E501


        :return: The utm_source of this UtmFields.  # noqa: E501
        :rtype: str
        """
        return self._utm_source

    @utm_source.setter
    def utm_source(self, utm_source):
        """Sets the utm_source of this UtmFields.


        :param utm_source: The utm_source of this UtmFields.  # noqa: E501
        :type: str
        """

        self._utm_source = utm_source

    @property
    def utm_campaign(self):
        """Gets the utm_campaign of this UtmFields.  # noqa: E501


        :return: The utm_campaign of this UtmFields.  # noqa: E501
        :rtype: str
        """
        return self._utm_campaign

    @utm_campaign.setter
    def utm_campaign(self, utm_campaign):
        """Sets the utm_campaign of this UtmFields.


        :param utm_campaign: The utm_campaign of this UtmFields.  # noqa: E501
        :type: str
        """

        self._utm_campaign = utm_campaign

    @property
    def utm_content(self):
        """Gets the utm_content of this UtmFields.  # noqa: E501


        :return: The utm_content of this UtmFields.  # noqa: E501
        :rtype: str
        """
        return self._utm_content

    @utm_content.setter
    def utm_content(self, utm_content):
        """Sets the utm_content of this UtmFields.


        :param utm_content: The utm_content of this UtmFields.  # noqa: E501
        :type: str
        """

        self._utm_content = utm_content

    @property
    def utm_medium(self):
        """Gets the utm_medium of this UtmFields.  # noqa: E501


        :return: The utm_medium of this UtmFields.  # noqa: E501
        :rtype: str
        """
        return self._utm_medium

    @utm_medium.setter
    def utm_medium(self, utm_medium):
        """Sets the utm_medium of this UtmFields.


        :param utm_medium: The utm_medium of this UtmFields.  # noqa: E501
        :type: str
        """

        self._utm_medium = utm_medium

    @property
    def referrer_url(self):
        """Gets the referrer_url of this UtmFields.  # noqa: E501


        :return: The referrer_url of this UtmFields.  # noqa: E501
        :rtype: str
        """
        return self._referrer_url

    @referrer_url.setter
    def referrer_url(self, referrer_url):
        """Sets the referrer_url of this UtmFields.


        :param referrer_url: The referrer_url of this UtmFields.  # noqa: E501
        :type: str
        """

        self._referrer_url = referrer_url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UtmFields):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, UtmFields):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional

from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_command
from anyscale.cli_logger import BlockLogger
from anyscale.schedule._private.schedule_sdk import PrivateScheduleSDK
from anyscale.schedule.models import ScheduleConfig, ScheduleState, ScheduleStatus


logger = BlockLogger()

_SCHEDULE_SDK_SINGLETON_KEY = "schedule_sdk"

_APPLY_EXAMPLE = """
import anyscale
from anyscale.job.models import JobConfig
from anyscale.schedule.models import ScheduleConfig

anyscale.schedule.apply(
    ScheduleConfig(
        cron_expression="0 0 * * * *",
        job_config=JobConfig(
            name="my-job",
            entrypoint="python main.py",
            working_dir=".",
        )
    )
)
"""

_APPLY_ARG_DOCSTRINGS = {"config": "The config options defining the schedule."}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_APPLY_EXAMPLE,
    arg_docstrings=_APPLY_ARG_DOCSTRINGS,
)
def apply(
    config: ScheduleConfig, *, _private_sdk: Optional[PrivateScheduleSDK] = None
) -> str:
    """Apply or update a schedule.

    Returns the id of the schedule.
    """
    return _private_sdk.apply(config)  # type: ignore


_SET_STATE_EXAMPLE = """
import anyscale
from anyscale.schedule.models import ScheduleState

anyscale.schedule.set_state(
    id="my=schedule-id",
    state=ScheduleState.DISABLED,
)
"""

_SET_STATE_ARG_DOCSTRINGS = {
    "id": "The id of the schedule.",
    "name": "The name of the schedule.",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "state": "The state to set the schedule to.",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_SET_STATE_EXAMPLE,
    arg_docstrings=_SET_STATE_ARG_DOCSTRINGS,
)
def set_state(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    state: ScheduleState,
    _private_sdk: Optional[PrivateScheduleSDK] = None,
) -> str:
    """Set the state of a schedule.

    Returns the id of the schedule.
    """
    return _private_sdk.set_state(  # type: ignore
        id=id, name=name, cloud=cloud, project=project, state=state,
    )


_STATUS_EXAMPLE = """
import anyscale
anyscale.schedule.status(id="cronjob_yt389jvskwht9k2ygx7rj6iz62")
"""

_STATUS_ARG_DOCSTRINGS = {
    "id": "The id of the schedule.",
    "name": "The name of the schedule.",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_STATUS_EXAMPLE,
    arg_docstrings=_STATUS_ARG_DOCSTRINGS,
)
def status(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateScheduleSDK] = None,
) -> ScheduleStatus:
    """Return the status of the schedule.
    """
    return _private_sdk.status(id=id, name=name, cloud=cloud, project=project)  # type: ignore


_TRIGGER_EXAMPLE = """
import anyscale
anyscale.schedule.trigger(id="cronjob_yt389jvskwht9k2ygx7rj6iz62")
"""

_TRIGGER_ARG_DOCSTRINGS = {
    "id": "The id of the schedule.",
    "name": "The name of the schedule.",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_TRIGGER_EXAMPLE,
    arg_docstrings=_TRIGGER_ARG_DOCSTRINGS,
)
def trigger(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateScheduleSDK] = None,
) -> str:
    """Trigger the execution of the schedule.
    """
    return _private_sdk.trigger(id=id, name=name, cloud=cloud, project=project)  # type: ignore


_LIST_EXAMPLE = """
import anyscale
from anyscale.schedule.models import ScheduleStatus

# List all schedules
for schedule in anyscale.schedule.list(max_items=10):
    print(f"{schedule.name}: {schedule.state}")

# Filter by project
schedules = list(anyscale.schedule.list(project="my-project"))
"""

_LIST_ARG_DOCSTRINGS = {
    "name": "Filter by schedule name.",
    "schedule_id": "Fetch a specific schedule by ID.",
    "project": "Filter by project name.",
    "cloud": "Filter by cloud name.",
    "creator_id": "Filter by creator ID.",
    "include_all_users": "Include schedules from all users.",
    "page_size": "Number of items per page.",
    "max_items": "Maximum total items to return.",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001, PLR0913
    *,
    name: Optional[str] = None,
    schedule_id: Optional[str] = None,
    project: Optional[str] = None,
    cloud: Optional[str] = None,
    creator_id: Optional[str] = None,
    include_all_users: bool = False,
    page_size: Optional[int] = None,
    max_items: Optional[int] = None,
    _private_sdk: Optional[PrivateScheduleSDK] = None,
) -> ResultIterator[ScheduleStatus]:
    """List schedules with filtering and pagination.

    Returns a ResultIterator that lazily fetches pages of schedules.
    """
    return _private_sdk.list(  # type: ignore
        name=name,
        schedule_id=schedule_id,
        project=project,
        cloud=cloud,
        creator_id=creator_id,
        include_all_users=include_all_users,
        page_size=page_size,
        max_items=max_items,
    )


_URL_EXAMPLE = """
import anyscale

# Get URL by ID
url = anyscale.schedule.url(id="cronjob_xxx")
print(url)

# Get URL by name
url = anyscale.schedule.url(name="my-schedule", cloud="my-cloud", project="my-project")
print(url)
"""

_URL_ARG_DOCSTRINGS = {
    "id": "The schedule ID.",
    "name": "The schedule name (requires cloud and project).",
    "cloud": "Cloud name (required with name).",
    "project": "Project name (required with name).",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_URL_EXAMPLE,
    arg_docstrings=_URL_ARG_DOCSTRINGS,
)
def url(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateScheduleSDK] = None,
) -> str:
    """Get the web UI URL for a schedule."""
    return _private_sdk.url(id=id, name=name, cloud=cloud, project=project)  # type: ignore


_DELETE_EXAMPLE = """
import anyscale

# Delete by ID
anyscale.schedule.delete(id="cronjob_xxx")

# Delete by name
anyscale.schedule.delete(name="my-schedule", cloud="my-cloud", project="my-project")
"""

_DELETE_ARG_DOCSTRINGS = {
    "id": "The schedule ID.",
    "name": "The schedule name (requires cloud and project).",
    "cloud": "Cloud name (required with name).",
    "project": "Project name (required with name).",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_DELETE_EXAMPLE,
    arg_docstrings=_DELETE_ARG_DOCSTRINGS,
)
def delete(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateScheduleSDK] = None,
) -> str:
    """Delete a schedule.

    If the schedule is active, it will be automatically paused before deletion.
    The schedule must have no active triggered jobs.
    Returns the ID of the deleted schedule.
    """
    return _private_sdk.delete(id=id, name=name, cloud=cloud, project=project)  # type: ignore



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class GCPConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'project_id': 'str',
        'host_project_id': 'str',
        'provider_name': 'str',
        'vpc_name': 'str',
        'subnet_names': 'list[str]',
        'firewall_policy_names': 'list[str]',
        'anyscale_service_account_email': 'str',
        'cluster_service_account_email': 'str',
        'memorystore_instance_name': 'str',
        'memorystore_endpoint': 'str',
        'deployment_manager_id': 'str'
    }

    attribute_map = {
        'project_id': 'project_id',
        'host_project_id': 'host_project_id',
        'provider_name': 'provider_name',
        'vpc_name': 'vpc_name',
        'subnet_names': 'subnet_names',
        'firewall_policy_names': 'firewall_policy_names',
        'anyscale_service_account_email': 'anyscale_service_account_email',
        'cluster_service_account_email': 'cluster_service_account_email',
        'memorystore_instance_name': 'memorystore_instance_name',
        'memorystore_endpoint': 'memorystore_endpoint',
        'deployment_manager_id': 'deployment_manager_id'
    }

    def __init__(self, project_id=None, host_project_id=None, provider_name=None, vpc_name=None, subnet_names=None, firewall_policy_names=None, anyscale_service_account_email=None, cluster_service_account_email=None, memorystore_instance_name=None, memorystore_endpoint=None, deployment_manager_id=None, local_vars_configuration=None):  # noqa: E501
        """GCPConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._project_id = None
        self._host_project_id = None
        self._provider_name = None
        self._vpc_name = None
        self._subnet_names = None
        self._firewall_policy_names = None
        self._anyscale_service_account_email = None
        self._cluster_service_account_email = None
        self._memorystore_instance_name = None
        self._memorystore_endpoint = None
        self._deployment_manager_id = None
        self.discriminator = None

        if project_id is not None:
            self.project_id = project_id
        if host_project_id is not None:
            self.host_project_id = host_project_id
        if provider_name is not None:
            self.provider_name = provider_name
        if vpc_name is not None:
            self.vpc_name = vpc_name
        if subnet_names is not None:
            self.subnet_names = subnet_names
        if firewall_policy_names is not None:
            self.firewall_policy_names = firewall_policy_names
        if anyscale_service_account_email is not None:
            self.anyscale_service_account_email = anyscale_service_account_email
        if cluster_service_account_email is not None:
            self.cluster_service_account_email = cluster_service_account_email
        if memorystore_instance_name is not None:
            self.memorystore_instance_name = memorystore_instance_name
        if memorystore_endpoint is not None:
            self.memorystore_endpoint = memorystore_endpoint
        if deployment_manager_id is not None:
            self.deployment_manager_id = deployment_manager_id

    @property
    def project_id(self):
        """Gets the project_id of this GCPConfig.  # noqa: E501

        The GCP project ID.  # noqa: E501

        :return: The project_id of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this GCPConfig.

        The GCP project ID.  # noqa: E501

        :param project_id: The project_id of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def host_project_id(self):
        """Gets the host_project_id of this GCPConfig.  # noqa: E501

        The host project ID for shared VPCs.  # noqa: E501

        :return: The host_project_id of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._host_project_id

    @host_project_id.setter
    def host_project_id(self, host_project_id):
        """Sets the host_project_id of this GCPConfig.

        The host project ID for shared VPCs.  # noqa: E501

        :param host_project_id: The host_project_id of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._host_project_id = host_project_id

    @property
    def provider_name(self):
        """Gets the provider_name of this GCPConfig.  # noqa: E501

        Workload Identity Federation provider name for Anyscale access.  # noqa: E501

        :return: The provider_name of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._provider_name

    @provider_name.setter
    def provider_name(self, provider_name):
        """Sets the provider_name of this GCPConfig.

        Workload Identity Federation provider name for Anyscale access.  # noqa: E501

        :param provider_name: The provider_name of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._provider_name = provider_name

    @property
    def vpc_name(self):
        """Gets the vpc_name of this GCPConfig.  # noqa: E501

        VPC name.  # noqa: E501

        :return: The vpc_name of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._vpc_name

    @vpc_name.setter
    def vpc_name(self, vpc_name):
        """Sets the vpc_name of this GCPConfig.

        VPC name.  # noqa: E501

        :param vpc_name: The vpc_name of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._vpc_name = vpc_name

    @property
    def subnet_names(self):
        """Gets the subnet_names of this GCPConfig.  # noqa: E501

        List of GCP subnet names.  # noqa: E501

        :return: The subnet_names of this GCPConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._subnet_names

    @subnet_names.setter
    def subnet_names(self, subnet_names):
        """Sets the subnet_names of this GCPConfig.

        List of GCP subnet names.  # noqa: E501

        :param subnet_names: The subnet_names of this GCPConfig.  # noqa: E501
        :type: list[str]
        """

        self._subnet_names = subnet_names

    @property
    def firewall_policy_names(self):
        """Gets the firewall_policy_names of this GCPConfig.  # noqa: E501

        List of GCP firewall policy names.  # noqa: E501

        :return: The firewall_policy_names of this GCPConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._firewall_policy_names

    @firewall_policy_names.setter
    def firewall_policy_names(self, firewall_policy_names):
        """Sets the firewall_policy_names of this GCPConfig.

        List of GCP firewall policy names.  # noqa: E501

        :param firewall_policy_names: The firewall_policy_names of this GCPConfig.  # noqa: E501
        :type: list[str]
        """

        self._firewall_policy_names = firewall_policy_names

    @property
    def anyscale_service_account_email(self):
        """Gets the anyscale_service_account_email of this GCPConfig.  # noqa: E501

        The Anyscale service account email.  # noqa: E501

        :return: The anyscale_service_account_email of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._anyscale_service_account_email

    @anyscale_service_account_email.setter
    def anyscale_service_account_email(self, anyscale_service_account_email):
        """Sets the anyscale_service_account_email of this GCPConfig.

        The Anyscale service account email.  # noqa: E501

        :param anyscale_service_account_email: The anyscale_service_account_email of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._anyscale_service_account_email = anyscale_service_account_email

    @property
    def cluster_service_account_email(self):
        """Gets the cluster_service_account_email of this GCPConfig.  # noqa: E501

        The service account email attached to Ray clusters.  # noqa: E501

        :return: The cluster_service_account_email of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._cluster_service_account_email

    @cluster_service_account_email.setter
    def cluster_service_account_email(self, cluster_service_account_email):
        """Sets the cluster_service_account_email of this GCPConfig.

        The service account email attached to Ray clusters.  # noqa: E501

        :param cluster_service_account_email: The cluster_service_account_email of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._cluster_service_account_email = cluster_service_account_email

    @property
    def memorystore_instance_name(self):
        """Gets the memorystore_instance_name of this GCPConfig.  # noqa: E501

        The Memorystore instance name.  # noqa: E501

        :return: The memorystore_instance_name of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._memorystore_instance_name

    @memorystore_instance_name.setter
    def memorystore_instance_name(self, memorystore_instance_name):
        """Sets the memorystore_instance_name of this GCPConfig.

        The Memorystore instance name.  # noqa: E501

        :param memorystore_instance_name: The memorystore_instance_name of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._memorystore_instance_name = memorystore_instance_name

    @property
    def memorystore_endpoint(self):
        """Gets the memorystore_endpoint of this GCPConfig.  # noqa: E501

        The Memorystore instance endpoint.  # noqa: E501

        :return: The memorystore_endpoint of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._memorystore_endpoint

    @memorystore_endpoint.setter
    def memorystore_endpoint(self, memorystore_endpoint):
        """Sets the memorystore_endpoint of this GCPConfig.

        The Memorystore instance endpoint.  # noqa: E501

        :param memorystore_endpoint: The memorystore_endpoint of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._memorystore_endpoint = memorystore_endpoint

    @property
    def deployment_manager_id(self):
        """Gets the deployment_manager_id of this GCPConfig.  # noqa: E501

        The deployment manager deployment ID, for Anyscale-managed resources.  # noqa: E501

        :return: The deployment_manager_id of this GCPConfig.  # noqa: E501
        :rtype: str
        """
        return self._deployment_manager_id

    @deployment_manager_id.setter
    def deployment_manager_id(self, deployment_manager_id):
        """Sets the deployment_manager_id of this GCPConfig.

        The deployment manager deployment ID, for Anyscale-managed resources.  # noqa: E501

        :param deployment_manager_id: The deployment_manager_id of this GCPConfig.  # noqa: E501
        :type: str
        """

        self._deployment_manager_id = deployment_manager_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GCPConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GCPConfig):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class GlobalWorkspaceTemplate(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'title': 'str',
        'description': 'str',
        'emoji': 'str',
        'labels': 'list[str]',
        'url': 'str',
        'maximum_uptime_minutes': 'int',
        'logo_ids': 'list[str]',
        'oa_group_name': 'str',
        'icon_type': 'str',
        'icon_bg_color': 'str',
        'mins_to_complete': 'int',
        'complexity': 'ComplexityLevel'
    }

    attribute_map = {
        'id': 'id',
        'title': 'title',
        'description': 'description',
        'emoji': 'emoji',
        'labels': 'labels',
        'url': 'url',
        'maximum_uptime_minutes': 'maximum_uptime_minutes',
        'logo_ids': 'logo_ids',
        'oa_group_name': 'oa_group_name',
        'icon_type': 'icon_type',
        'icon_bg_color': 'icon_bg_color',
        'mins_to_complete': 'mins_to_complete',
        'complexity': 'complexity'
    }

    def __init__(self, id=None, title=None, description=None, emoji=None, labels=None, url=None, maximum_uptime_minutes=None, logo_ids=None, oa_group_name=None, icon_type=None, icon_bg_color=None, mins_to_complete=None, complexity=None, local_vars_configuration=None):  # noqa: E501
        """GlobalWorkspaceTemplate - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._title = None
        self._description = None
        self._emoji = None
        self._labels = None
        self._url = None
        self._maximum_uptime_minutes = None
        self._logo_ids = None
        self._oa_group_name = None
        self._icon_type = None
        self._icon_bg_color = None
        self._mins_to_complete = None
        self._complexity = None
        self.discriminator = None

        self.id = id
        self.title = title
        self.description = description
        self.emoji = emoji
        self.labels = labels
        self.url = url
        if maximum_uptime_minutes is not None:
            self.maximum_uptime_minutes = maximum_uptime_minutes
        self.logo_ids = logo_ids
        if oa_group_name is not None:
            self.oa_group_name = oa_group_name
        if icon_type is not None:
            self.icon_type = icon_type
        if icon_bg_color is not None:
            self.icon_bg_color = icon_bg_color
        if mins_to_complete is not None:
            self.mins_to_complete = mins_to_complete
        if complexity is not None:
            self.complexity = complexity

    @property
    def id(self):
        """Gets the id of this GlobalWorkspaceTemplate.  # noqa: E501

        The id of the workspace template.  # noqa: E501

        :return: The id of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this GlobalWorkspaceTemplate.

        The id of the workspace template.  # noqa: E501

        :param id: The id of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def title(self):
        """Gets the title of this GlobalWorkspaceTemplate.  # noqa: E501

        The title of the workspace template  # noqa: E501

        :return: The title of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this GlobalWorkspaceTemplate.

        The title of the workspace template  # noqa: E501

        :param title: The title of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and title is None:  # noqa: E501
            raise ValueError("Invalid value for `title`, must not be `None`")  # noqa: E501

        self._title = title

    @property
    def description(self):
        """Gets the description of this GlobalWorkspaceTemplate.  # noqa: E501

        The description of the workspace template  # noqa: E501

        :return: The description of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this GlobalWorkspaceTemplate.

        The description of the workspace template  # noqa: E501

        :param description: The description of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and description is None:  # noqa: E501
            raise ValueError("Invalid value for `description`, must not be `None`")  # noqa: E501

        self._description = description

    @property
    def emoji(self):
        """Gets the emoji of this GlobalWorkspaceTemplate.  # noqa: E501

        The emoji of the workspace template  # noqa: E501

        :return: The emoji of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._emoji

    @emoji.setter
    def emoji(self, emoji):
        """Sets the emoji of this GlobalWorkspaceTemplate.

        The emoji of the workspace template  # noqa: E501

        :param emoji: The emoji of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and emoji is None:  # noqa: E501
            raise ValueError("Invalid value for `emoji`, must not be `None`")  # noqa: E501

        self._emoji = emoji

    @property
    def labels(self):
        """Gets the labels of this GlobalWorkspaceTemplate.  # noqa: E501

        The labels of the workspace template  # noqa: E501

        :return: The labels of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: list[str]
        """
        return self._labels

    @labels.setter
    def labels(self, labels):
        """Sets the labels of this GlobalWorkspaceTemplate.

        The labels of the workspace template  # noqa: E501

        :param labels: The labels of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: list[str]
        """
        if self.local_vars_configuration.client_side_validation and labels is None:  # noqa: E501
            raise ValueError("Invalid value for `labels`, must not be `None`")  # noqa: E501

        self._labels = labels

    @property
    def url(self):
        """Gets the url of this GlobalWorkspaceTemplate.  # noqa: E501

        The url of the workspace template  # noqa: E501

        :return: The url of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this GlobalWorkspaceTemplate.

        The url of the workspace template  # noqa: E501

        :param url: The url of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and url is None:  # noqa: E501
            raise ValueError("Invalid value for `url`, must not be `None`")  # noqa: E501

        self._url = url

    @property
    def maximum_uptime_minutes(self):
        """Gets the maximum_uptime_minutes of this GlobalWorkspaceTemplate.  # noqa: E501

        The maximum uptime of the workspace in minutes. The workspace will force terminate after this time.  # noqa: E501

        :return: The maximum_uptime_minutes of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: int
        """
        return self._maximum_uptime_minutes

    @maximum_uptime_minutes.setter
    def maximum_uptime_minutes(self, maximum_uptime_minutes):
        """Sets the maximum_uptime_minutes of this GlobalWorkspaceTemplate.

        The maximum uptime of the workspace in minutes. The workspace will force terminate after this time.  # noqa: E501

        :param maximum_uptime_minutes: The maximum_uptime_minutes of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: int
        """

        self._maximum_uptime_minutes = maximum_uptime_minutes

    @property
    def logo_ids(self):
        """Gets the logo_ids of this GlobalWorkspaceTemplate.  # noqa: E501

        The ids for the workspace template logos that we show on the FE  # noqa: E501

        :return: The logo_ids of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: list[str]
        """
        return self._logo_ids

    @logo_ids.setter
    def logo_ids(self, logo_ids):
        """Sets the logo_ids of this GlobalWorkspaceTemplate.

        The ids for the workspace template logos that we show on the FE  # noqa: E501

        :param logo_ids: The logo_ids of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: list[str]
        """
        if self.local_vars_configuration.client_side_validation and logo_ids is None:  # noqa: E501
            raise ValueError("Invalid value for `logo_ids`, must not be `None`")  # noqa: E501

        self._logo_ids = logo_ids

    @property
    def oa_group_name(self):
        """Gets the oa_group_name of this GlobalWorkspaceTemplate.  # noqa: E501

        The name of the OA group for the workspace template  # noqa: E501

        :return: The oa_group_name of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._oa_group_name

    @oa_group_name.setter
    def oa_group_name(self, oa_group_name):
        """Sets the oa_group_name of this GlobalWorkspaceTemplate.

        The name of the OA group for the workspace template  # noqa: E501

        :param oa_group_name: The oa_group_name of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """

        self._oa_group_name = oa_group_name

    @property
    def icon_type(self):
        """Gets the icon_type of this GlobalWorkspaceTemplate.  # noqa: E501

        The type of the icon for the workspace template  # noqa: E501

        :return: The icon_type of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._icon_type

    @icon_type.setter
    def icon_type(self, icon_type):
        """Sets the icon_type of this GlobalWorkspaceTemplate.

        The type of the icon for the workspace template  # noqa: E501

        :param icon_type: The icon_type of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """

        self._icon_type = icon_type

    @property
    def icon_bg_color(self):
        """Gets the icon_bg_color of this GlobalWorkspaceTemplate.  # noqa: E501

        The background color of the icon for the workspace template  # noqa: E501

        :return: The icon_bg_color of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._icon_bg_color

    @icon_bg_color.setter
    def icon_bg_color(self, icon_bg_color):
        """Sets the icon_bg_color of this GlobalWorkspaceTemplate.

        The background color of the icon for the workspace template  # noqa: E501

        :param icon_bg_color: The icon_bg_color of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: str
        """

        self._icon_bg_color = icon_bg_color

    @property
    def mins_to_complete(self):
        """Gets the mins_to_complete of this GlobalWorkspaceTemplate.  # noqa: E501

        The number of minutes expected to complete the template  # noqa: E501

        :return: The mins_to_complete of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: int
        """
        return self._mins_to_complete

    @mins_to_complete.setter
    def mins_to_complete(self, mins_to_complete):
        """Sets the mins_to_complete of this GlobalWorkspaceTemplate.

        The number of minutes expected to complete the template  # noqa: E501

        :param mins_to_complete: The mins_to_complete of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: int
        """

        self._mins_to_complete = mins_to_complete

    @property
    def complexity(self):
        """Gets the complexity of this GlobalWorkspaceTemplate.  # noqa: E501

        The complexity of the workspace template  # noqa: E501

        :return: The complexity of this GlobalWorkspaceTemplate.  # noqa: E501
        :rtype: ComplexityLevel
        """
        return self._complexity

    @complexity.setter
    def complexity(self, complexity):
        """Sets the complexity of this GlobalWorkspaceTemplate.

        The complexity of the workspace template  # noqa: E501

        :param complexity: The complexity of this GlobalWorkspaceTemplate.  # noqa: E501
        :type: ComplexityLevel
        """

        self._complexity = complexity

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GlobalWorkspaceTemplate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, GlobalWorkspaceTemplate):
            return True

        return self.to_dict() != other.to_dict()



from typing import Any, cast, Dict, List, Optional, Tuple, Union

import click
from click import ClickException
from openapi_client.rest import ApiException

from anyscale.authenticate import get_auth_api_client
from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.api.default_api import DefaultApi
from anyscale.client.openapi_client.models import (
    CreateCloudResource,
    CreateCloudResourceGCP,
)
from anyscale.sdk.anyscale_client.api.default_api import DefaultApi as SDKDefaultApi
from anyscale.sdk.anyscale_client.models.cloud import Cloud


log = BlockLogger()


def get_cloud_json_from_id(cloud_id: str, api_client: DefaultApi) -> Dict["str", Any]:
    try:
        cloud = api_client.get_cloud_api_v2_clouds_cloud_id_get(
            cloud_id=cloud_id
        ).result
    except ApiException:
        return {
            "error": {
                "cloud_id": cloud_id,
                "message": f"The cloud with id, {cloud_id} has been deleted. Please create a new cloud with `anyscale cloud setup`.",
            }
        }
    return {
        "id": cloud.id,
        "name": cloud.name,
        "provider": cloud.provider,
        "region": cloud.region,
        "credentials": cloud.credentials,
        "config": cloud.config,
        "state": cloud.state,
    }


def get_cloud_id_and_name(
    api_client: Optional[DefaultApi] = None,
    cloud_id: Optional[str] = None,
    cloud_name: Optional[str] = None,
) -> Tuple[str, str]:
    if api_client is None:
        api_client = get_auth_api_client().api_client
    if not cloud_id and not cloud_name:
        raise ClickException("Must either provide the cloud name or cloud id.")
    if cloud_id and cloud_name:
        raise ClickException(
            "Both '--cloud-id' and '--cloud-name' specified. Please only use one."
        )
    elif cloud_name:
        resp_get_cloud = api_client.find_cloud_by_name_api_v2_clouds_find_by_name_post(
            cloud_name_options={"name": cloud_name}
        )
        cloud = resp_get_cloud.result
    elif cloud_id:
        resp_get_cloud = api_client.get_cloud_api_v2_clouds_cloud_id_get(
            cloud_id=cloud_id
        )

        cloud = resp_get_cloud.result
    else:
        try:
            clouds = api_client.list_clouds_api_v2_clouds_get().results
        except Exception as e:
            if (isinstance(e, ApiException) and e.status == 404) or (  # type: ignore
                isinstance(e, ClickException) and "Reason: 404" in str(e)
            ):
                # No clouds
                raise ClickException(
                    'There are no clouds assigned to your account. Please create a cloud using "anyscale cloud setup".'
                )
            raise e

        if len(clouds) == 0:
            # No clouds
            raise ClickException(
                'There are no clouds assigned to your account. Please create a cloud using "anyscale cloud setup".'
            )

        if len(clouds) > 1:
            raise ClickException(
                "Multiple clouds: {}\n"
                "Please specify the one you want to refer to using --cloud-name.".format(
                    [cloud.name for cloud in clouds]
                )
            )
        cloud = clouds[0]
    return cloud.id, cloud.name


def get_cloud_resource_id_by_name(
    cloud_id: str, cloud_resource_name: str, api_client: Optional[DefaultApi] = None,
) -> str:
    if api_client is None:
        api_client = get_auth_api_client().api_client

    cloud_resource = api_client.find_cloud_resource_by_name_api_v2_clouds_cloud_id_find_cloud_resource_by_name_post(
        cloud_id=cloud_id, cloud_resource_name=cloud_resource_name,
    ).result
    return cloud_resource.cloud_resource_id


def get_cloud_resource_by_cloud_id(
    cloud_id: str, cloud_provider: str, anyscale_api_client: Optional[DefaultApi] = None
) -> Union[CreateCloudResource, CreateCloudResourceGCP]:
    if anyscale_api_client is None:
        anyscale_api_client = get_auth_api_client().anyscale_api_client
    assert anyscale_api_client is not None
    cloud_resource = None
    if cloud_provider == "AWS":
        cloud = anyscale_api_client.get_cloud_with_cloud_resource_api_v2_clouds_with_cloud_resource_router_cloud_id_get(
            cloud_id
        ).result
        cloud_resource = cloud.cloud_resource
    elif cloud_provider == "GCP":
        cloud = anyscale_api_client.get_cloud_with_cloud_resource_api_v2_clouds_with_cloud_resource_gcp_router_cloud_id_get(
            cloud_id
        ).result
        cloud_resource = cloud.cloud_resource
    return cloud_resource


def get_last_used_cloud(
    project_id: Optional[str],
    anyscale_api_client: Optional[Union[DefaultApi, SDKDefaultApi]] = None,
) -> str:
    """Return the name of the cloud last used in the project.

    Args:
        project_id (str): The project to get the last used cloud for.

    Returns:
        Name of the cloud last used in this project.

    TODO(nikita): Condense with anyscale.connect after unifying logging.
    TODO: get_selected_cloud_id_or_default should be the preferred way of
    getting a cloud in CLI code.
    """
    if anyscale_api_client is None:
        anyscale_api_client = get_auth_api_client().anyscale_api_client
    if project_id:
        cloud_id = anyscale_api_client.get_project(project_id).result.last_used_cloud_id  # type: ignore
    else:
        cloud_id = None
    if cloud_id:
        try:
            cloud = anyscale_api_client.get_cloud(cloud_id).result  # type: ignore
        except Exception:  # noqa: BLE001
            raise click.ClickException(
                f"Failed to fetch Cloud with id: {cloud_id}. Please specify `cloud` in the command."
            )
    else:
        # TODO(nikita): Add some logics or checks here so users don't start clusters with clouds
        # they don't intend to.
        clouds = get_all_clouds(anyscale_api_client)
        if len(clouds) > 0:
            # Clouds are sorted in descending order, pick the oldest one as default.
            cloud = clouds[-1]
        else:
            raise click.ClickException(
                "No cloud configured, please set up a cloud with 'anyscale cloud setup'."
            )

    cloud_name = cloud.name
    log.debug(
        f"Using last active cloud '{cloud_name}'. "
        "Specify `cloud` in the command to overwrite."
    )
    return cast(str, cloud_name)


def get_all_clouds(
    anyscale_api_client: Optional[Union[DefaultApi, SDKDefaultApi]] = None,
) -> List[Cloud]:
    """Fetches all Clouds the user has access to.
    Returns:
        List of all Clouds the user has access to.

    TODO(nikita): Condense with anyscale.connect after unifying logging.
    """
    if anyscale_api_client is None:
        anyscale_api_client = get_auth_api_client().anyscale_api_client

    cloud_list_response = anyscale_api_client.search_clouds({"paging": {"count": 50}})  # type: ignore
    all_clouds = cloud_list_response.results
    next_paging_token = cloud_list_response.metadata.next_paging_token

    while next_paging_token:
        cloud_list_response = anyscale_api_client.search_clouds(  # type: ignore
            {"paging": {"count": 50, "paging_token": next_paging_token}}
        )
        next_paging_token = cloud_list_response.metadata.next_paging_token
        all_clouds.extend(cloud_list_response.results)

    return all_clouds  # type: ignore


def get_organization_default_cloud(api_client: DefaultApi) -> Optional[str]:
    """Return default cloud name for organization if it exists and
    if user has correct permissions for it.
    Returns:
        Name of default cloud name for organization if it exists and
        if user has correct permissions for it.
    """
    user = api_client.get_user_info_api_v2_userinfo_get().result
    organization = user.organizations[0]  # Each user only has one org
    if organization.default_cloud_id:
        try:
            # Check permissions
            _, cloud_name = get_cloud_id_and_name(
                api_client, cloud_id=organization.default_cloud_id
            )
            return str(cloud_name)
        except Exception:  # noqa: BLE001
            return None
    return None


def get_organization_id(api_client: DefaultApi) -> str:
    user = api_client.get_user_info_api_v2_userinfo_get().result
    if user.organizations:
        organization = user.organizations[0]  # Each user only has one org
        return organization.id
    raise click.ClickException(
        "No organization found. Please contact Anyscale support for help."
    )



import ipaddress
import re
from typing import Iterable, List, Optional, Tuple

import click
from click import ClickException
from google.api_core.exceptions import NotFound, PermissionDenied
from google.cloud import compute_v1
from google.cloud.storage.bucket import Bucket
from google.cloud.storage.constants import (
    PUBLIC_ACCESS_PREVENTION_ENFORCED,
    PUBLIC_ACCESS_PREVENTION_INHERITED,
)
from google.iam.v1.policy_pb2 import Binding
from googleapiclient.errors import HttpError

from anyscale.cli_logger import CloudSetupLogger
from anyscale.client.openapi_client.models import CloudAnalyticsEventCloudResource
from anyscale.shared_anyscale_utils.conf import ANYSCALE_CORS_ORIGIN
from anyscale.utils.cloud_utils import CloudSetupError
from anyscale.utils.gcp_managed_setup_utils import enable_project_apis
from anyscale.utils.gcp_utils import (
    binding_from_dictionary,
    check_policy_bindings,
    check_required_policy_bindings,
    GCP_REQUIRED_APIS,
    GCPLogger,
    GoogleCloudClientFactory,
)
from anyscale.utils.network_verification import (
    check_inbound_firewall_permissions,
    FirewallRule,
    GCP_SUBNET_CAPACITY,
    Protocol,
)


# Refer to https://cloud.google.com/python/docs/reference/file/latest/google.cloud.filestore_v1.
# services.cloud_filestore_manager.CloudFilestoreManagerClient, filestore name is in the format
# projects/{project_id}/locations/{location}/instances/{instance_id}.
_FILESTORE_NAME_REGEX_PATTERN = r"projects/(?P<project_id>[^/]+)/locations/(?P<location>[^/]+)/instances/(?P<instance_id>[^/]+)"

# A subnet with this purpose must be in the VPC for private load balancers to work.
# See: https://cloud.google.com/load-balancing/docs/proxy-only-subnets
PROXY_ONLY_SUBNET_PURPOSE = "REGIONAL_MANAGED_PROXY"


def check_gcp_cors_rules(cors_rules: List[dict]) -> Tuple[bool, str]:
    """
    Check if GCP GCS CORS rules are correctly configured for Anyscale.

    This is a shared helper used by both cloud verification and CORS update utilities.

    Args:
        cors_rules: List of CORS config dictionaries from GCS bucket

    Returns:
        Tuple of (is_correct, reason)
    """
    if not cors_rules:
        return False, "No CORS configuration exists"

    for cors_config in cors_rules:
        if (
            ANYSCALE_CORS_ORIGIN in cors_config.get("origin", [])
            and "*" in cors_config.get("responseHeader", [])
            and "GET" in cors_config.get("method", [])
        ):
            return True, "CORS already configured correctly"

    return False, "CORS missing required responseHeader for file viewer"


def verify_gcp_networking(  # noqa: PLR0911, PLR0913
    factory: GoogleCloudClientFactory,
    vpc_name: Optional[str],
    subnet_ids: Optional[List[str]],
    project_id: str,
    cloud_region: str,
    logger: GCPLogger,
    strict: bool = False,
    is_private_service_cloud: bool = False,
    ignore_capacity_errors: bool = False,
) -> bool:
    """Verify the existence and connectedness of the VPC & Subnet."""
    # TODO Verify Internet Gateway
    if not vpc_name:
        logger.internal.warning("No VPC provided. Please provide a VPC.")
        return False

    try:
        vpc = factory.compute_v1.NetworksClient().get(
            project=project_id, network=vpc_name
        )
    except NotFound:
        logger.log_resource_not_found_error("VPC", vpc_name, project_id)
        return False

    if not subnet_ids:
        logger.internal.warning(
            "No subnets provided. Please provide at least one subnet."
        )
        return False

    subnet_name = subnet_ids[0]  # TODO (congding): multiple subnets provided
    if len(subnet_ids) > 1:
        logger.internal.warning(
            "Multiple subnets provided. Only taking the first subnet and ignoring the rest."
        )
        if strict:
            return False
    try:
        subnet = factory.compute_v1.SubnetworksClient().get(
            project=project_id, subnetwork=subnet_name, region=cloud_region,
        )
    except NotFound:
        logger.log_resource_not_found_error("Subnet", subnet_name, project_id)
        return False

    if subnet.network != vpc.self_link:
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_SUBNET,
            CloudSetupError.SUBNET_NOT_IN_VPC,
        )
        logger.internal.error(f"Subnet {subnet_name} is not part of {vpc_name}!")
        return False

    if is_private_service_cloud and not _get_proxy_only_subnet_in_vpc(
        factory, project_id, cloud_region, vpc_name
    ):
        logger.internal.warning(
            f"Could not find a subnet with purpose `{PROXY_ONLY_SUBNET_PURPOSE}` in the VPC {vpc_name}. A proxy-only subnet is required to run private Anyscale Services in your account. See https://cloud.google.com/load-balancing/docs/proxy-only-subnets for more details."
        )
        if strict:
            return False

    return _gcp_subnet_has_enough_capacity(
        subnet, logger.internal, ignore_capacity_errors
    )


def _gcp_subnet_has_enough_capacity(
    subnet: compute_v1.types.compute.Subnetwork,
    logger: CloudSetupLogger,
    ignore_capacity_errors: bool = False,
) -> bool:
    """Verify if the subnet provided has a large enough IP address block."""
    return ignore_capacity_errors or bool(
        GCP_SUBNET_CAPACITY.verify_network_capacity(
            cidr_block_str=subnet.ip_cidr_range,
            resource_name=subnet.name,
            logger=logger,
        )
    )


def _get_proxy_only_subnet_in_vpc(
    factory: GoogleCloudClientFactory,
    project_id: str,
    cloud_region: str,
    vpc_name: str,
) -> Optional[compute_v1.types.compute.Subnetwork]:
    # Check if there exists a subnet in the VPC and region with purpose `REGIONAL_MANAGED_PROXY`
    subnets = factory.compute_v1.SubnetworksClient().list(
        project=project_id, region=cloud_region
    )
    for subnet in subnets:
        subnet_vpc = subnet.network.split("/")[-1]
        if vpc_name == subnet_vpc and subnet.purpose == PROXY_ONLY_SUBNET_PURPOSE:
            return subnet
    return None


def verify_gcp_project(  # noqa: PLR0911
    factory: GoogleCloudClientFactory,
    project_id: str,
    enable_memorystore_api: bool,
    logger: GCPLogger,
    strict: bool = False,
) -> bool:
    """Verify if the project exists and is active.

    NOTE: This also checks that Compute Engine Service Agent is configured in the default way.
    """
    project_client = factory.resourcemanager_v3.ProjectsClient()
    try:
        project = project_client.get_project(name=f"projects/{project_id}")
    except (NotFound, PermissionDenied):
        logger.log_resource_not_found_error("Project", project_id)
        return False
    if project.state != project.State.ACTIVE:
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_PROJECT,
            CloudSetupError.PROJECT_NOT_ACTIVE,
        )
        logger.internal.error(
            f"Project {project_id} is in state: {project.state}, not active"
        )
        return False

    iam_policies = project_client.get_iam_policy(resource=f"projects/{project_id}")
    project_number = project.name.split("/")[1]

    if not check_policy_bindings(
        iam_policies.bindings,
        f"serviceAccount:service-{project_number}@compute-system.iam.gserviceaccount.com",
        {"roles/compute.serviceAgent"},
    ):
        logger.internal.warning(
            "The Compute Engine Service Agent does not have the standard IAM Role of 'roles/compute.serviceAgent' in your project.\n"
            "This is not recommended by Google and may result in an inability for Compute Engine to function properly:\n"
            "https://cloud.google.com/compute/docs/access/service-accounts#compute_engine_service_account"
        )
        if strict:
            return False

    # Verify that APIs are Enabled
    service_usage_client = factory.build("serviceusage", "v1")
    if enable_memorystore_api:
        apis = GCP_REQUIRED_APIS + ["redis.googleapis.com"]  # Memorystore for Redis
    else:
        apis = GCP_REQUIRED_APIS
    response = (
        service_usage_client.services()
        .batchGet(
            parent=f"projects/{project_id}",
            names=[f"projects/{project_id}/services/{api}" for api in apis],
        )
        .execute()
    )

    unenabled_apis = [
        service["config"]["name"]
        for service in response["services"]
        if service["state"] != "ENABLED"
    ]
    if len(unenabled_apis) == 0:
        return True
    logger.spinner.stop()
    if not logger.yes and not click.confirm(
        f"The following APIs are not enabled in your project: {unenabled_apis}\n"
        "Anyscale won't function properly if you don't enable these APIs.\n"
        "Would you like to enable these APIs and proceed?"
    ):
        logger.spinner.start()
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_PROJECT,
            CloudSetupError.API_NOT_ENABLED,
        )
        return False

    spinner_text = logger.spinner.text
    logger.spinner.text = "Enabling APIs..."
    logger.spinner.start()

    # enable all required apis, the operation is idompotent
    try:
        enable_project_apis(
            factory, project_id, logger.internal, enable_memorystore_api
        )
    except ClickException as e:
        logger.internal.error(e.message)
        logger.spinner.text = spinner_text
        return False

    # APIs are enabled successfully
    logger.spinner.text = spinner_text
    return True


def verify_gcp_access_service_account(
    factory: GoogleCloudClientFactory,
    anyscale_access_service_account: Optional[str],
    project_id: str,
    logger: GCPLogger,
) -> bool:
    """Verify if the Service Account meant to grant Anyscale access to this cloud has access to the specified project.

    NOTE: We verify that this service account can call signBlob on itself because this is necessary for downloading logs.
    """
    service_account_client = factory.build("iam", "v1").projects().serviceAccounts()
    try:
        service_account_iam_policy = service_account_client.getIamPolicy(
            resource=f"projects/-/serviceAccounts/{anyscale_access_service_account}"
        ).execute()
    except HttpError as e:
        if e.status_code == 404:
            logger.log_resource_not_found_error(
                "Anyscale Access Service Account", anyscale_access_service_account
            )
        else:
            logger.internal.log_resource_exception(
                CloudAnalyticsEventCloudResource.GCP_SERVICE_ACCOUNT, e
            )
            logger.internal.error(str(e))
        return False

    if service_account_iam_policy.get("bindings") is None or not check_policy_bindings(
        binding_from_dictionary(service_account_iam_policy["bindings"]),
        f"serviceAccount:{anyscale_access_service_account}",
        {"roles/iam.serviceAccountTokenCreator"},
    ):
        logger.confirm_missing_permission(
            f"Service Account {anyscale_access_service_account} must have the `iam.serviceAccounts.signBlob` permission to perform log download from the Anyscale platform.\n"
            "Please grant the `roles/iam.serviceAccountTokenCreator` role to this Service Account to pass this check."
        )

    project_client = factory.resourcemanager_v3.ProjectsClient()
    iam_policies = project_client.get_iam_policy(resource=f"projects/{project_id}")
    if not check_policy_bindings(
        iam_policies.bindings,
        f"serviceAccount:{anyscale_access_service_account}",
        {"roles/editor", "roles/owner"},
    ):
        logger.confirm_missing_permission(
            f"Service Account {anyscale_access_service_account} must have either `roles/editor` or `roles/owner` roles on the Project {project_id}.\n"
            "A more fine-grained set of permissions may be possible, but may break Anyscale's functionality."
        )

    return True


def verify_gcp_dataplane_service_account(
    factory: GoogleCloudClientFactory,
    service_account: Optional[str],
    project_id: str,
    logger: GCPLogger,
    strict: bool = False,
) -> bool:
    """Verify that the Service Account for compute instances exists *in* the specified project.

    Compute Engine's ability to use this role
    This relies on the fact that Compute Engine Service Agent has the roles/compute.serviceAgent Role
    """
    service_account_client = factory.build("iam", "v1").projects().serviceAccounts()
    try:
        resp = service_account_client.get(
            name=f"projects/-/serviceAccounts/{service_account}"
        ).execute()
    except HttpError as e:
        if e.status_code == 404:
            logger.log_resource_not_found_error(
                "Dataplane Service Account", service_account
            )
        else:
            logger.internal.log_resource_exception(
                CloudAnalyticsEventCloudResource.GCP_SERVICE_ACCOUNT, e
            )
            logger.internal.error(str(e))
        return False

    if resp["projectId"] != project_id:
        logger.internal.warning(
            "Service Account {} is in project {}, not {}. Please ensure that `constraints/iam.disableCrossProjectServiceAccountUsage` is not enforced.\n"
            "See: https://cloud.google.com/iam/docs/attach-service-accounts#enabling-cross-project for more information.".format(
                service_account, resp["projectId"], project_id
            )
        )
        if strict:
            return False

    project_client = factory.resourcemanager_v3.ProjectsClient()
    project_iam_bindings = project_client.get_iam_policy(
        resource=f"projects/{project_id}"
    )
    if not check_policy_bindings(
        project_iam_bindings.bindings,
        f"serviceAccount:{service_account}",
        {"roles/artifactregistry.reader", "roles/viewer"},
    ):
        logger.internal.warning(
            f"The dataplane Service Account {service_account} does not have `roles/artifactregistry.reader` or `roles/viewer` on {project_id}.\n"
            "This is safe to ignore if you are not using your own container images or are configuring access a different way."
        )
        if strict:
            return False

    return True


def verify_firewall_policy(  # noqa: PLR0911, PLR0912, C901, PLR0913
    factory: GoogleCloudClientFactory,
    firewall_policy_ids: Optional[List[str]],
    vpc_name: Optional[str],
    subnet_ids: Optional[List[str]],
    project_id: str,
    cloud_region: str,
    use_shared_vpc: bool,
    is_private_service_cloud: bool,
    logger: GCPLogger,
    strict: bool = False,
) -> bool:
    """Checks if the given firewall exists at either the Global or Regional level."""
    if not firewall_policy_ids:
        logger.internal.warning(
            "No firewall policies provided. Please provide at least one firewall policy."
        )
        return False

    firewall_policy = firewall_policy_ids[
        0
    ]  # TODO (congding): multiple firewall ids provided
    if len(firewall_policy_ids) > 1:
        logger.internal.warning(
            "Multiple firewall policies provided. Only taking the first firewall policy and ignoring the rest."
        )
        if strict:
            return False

    firewall = compute_v1.types.compute.FirewallPolicy()
    try:
        firewall = factory.compute_v1.NetworkFirewallPoliciesClient().get(
            project=project_id, firewall_policy=firewall_policy
        )
    except NotFound:
        logger.internal.info(
            f"Global firweall policy {firewall_policy} not found, trying in region: {cloud_region}."
        )
    if not firewall:
        try:
            firewall = factory.compute_v1.RegionNetworkFirewallPoliciesClient().get(
                project=project_id, firewall_policy=firewall_policy, region=cloud_region
            )
        except NotFound:
            logger.log_resource_not_found_error(
                "Firewall Policy", firewall_policy, project_id
            )
            return False

    if not any(
        association.attachment_target.split("/")[-1] == vpc_name
        for association in firewall.associations
    ):
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_FIREWALL_POLICY,
            CloudSetupError.FIREWALL_NOT_ASSOCIATED_WITH_VPC,
        )
        logger.internal.error(
            "Firewall policy {} is not associated with the VPC {}, but is associated with the following VPCs: {}".format(
                firewall_policy, vpc_name, firewall.associations
            )
        )
        return False

    if not subnet_ids:
        logger.internal.warning(
            "No subnets provided. Please provide at least one subnet."
        )
        return False

    subnet_obj = factory.compute_v1.SubnetworksClient().get(
        project=project_id, subnetwork=subnet_ids[0], region=cloud_region
    )  # TODO (congding): multiple subnets provided
    subnet = ipaddress.ip_network(subnet_obj.ip_cidr_range)

    rules = _firewall_rules_from_proto_resp(firewall.rules)
    if not check_inbound_firewall_permissions(
        rules,
        Protocol.tcp,
        {22},
        ipaddress.ip_network("0.0.0.0/0"),
        check_ingress_source_range=False,
    ):
        logger.internal.warning(
            "Firewall policy does not allow inbound access on port 22. Certain features may be unavailable."
        )
        if strict:
            return False
    if not check_inbound_firewall_permissions(
        rules,
        Protocol.tcp,
        {443},
        ipaddress.ip_network("0.0.0.0/0"),
        check_ingress_source_range=False,
    ):
        logger.internal.warning(
            "Firewall policy does not allow inbound access on port 443. Accessing features like Jupyter & Ray Dashboard may not work."
        )
        if strict:
            return False

    # If shared VPC is used, we need to check the firewall rule for Anyscale services is added
    # Since our service account cannot update the firewall policy in host project, so we need to make sure the rule is added during cloud setup
    if use_shared_vpc and not (
        check_inbound_firewall_permissions(
            rules, Protocol.tcp, {8000}, ipaddress.ip_network("35.191.0.0/16")
        )
        and check_inbound_firewall_permissions(
            rules, Protocol.tcp, {8000}, ipaddress.ip_network("130.211.0.0/22")
        )
    ):
        logger.internal.warning(
            "Firewall policy does not allow inbound access on port 8000 from '35.191.0.0/16' and '130.211.0.0/22'. "
            "This is required for Anyscale services to work correctly. "
            "See https://cloud.google.com/load-balancing/docs/health-checks#fw-rule for more details."
        )
        if strict:
            return False

    if use_shared_vpc and is_private_service_cloud:
        if not vpc_name:
            logger.internal.warning("No VPC provided. Please provide a VPC.")
            return False

        proxy_only_subnet = _get_proxy_only_subnet_in_vpc(
            factory, project_id, cloud_region, vpc_name
        )
        if not proxy_only_subnet:
            logger.internal.warning(
                f"Could not find a subnet with purpose `{PROXY_ONLY_SUBNET_PURPOSE}` in the VPC {vpc_name}. A proxy-only subnet is required to run private Anyscale Services in your account. See https://cloud.google.com/load-balancing/docs/proxy-only-subnets for more details."
            )
            if strict:
                return False
        elif not check_inbound_firewall_permissions(
            rules,
            Protocol.tcp,
            {8000},
            ipaddress.ip_network(proxy_only_subnet.ip_cidr_range),
        ):
            logger.internal.warning(
                "Firewall policy does not allow inbound access from the proxy-only subnet on port 8000. "
                "This is required for private Anyscale services to work correctly. "
            )
            if strict:
                return False

    if not check_inbound_firewall_permissions(rules, Protocol.tcp, None, subnet):
        logger.internal.error(
            "Firewall policy does not allow for internal communication, see https://cloud.google.com/vpc/docs/using-firewalls#common-use-cases-allow-internal for how to configure such a rule."
        )
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_FIREWALL_POLICY,
            CloudSetupError.INTERNAL_COMMUNICATION_NOT_ALLOWED,
        )
        return False
    return True


def _firewall_rules_from_proto_resp(
    rules: Iterable[compute_v1.types.compute.FirewallPolicyRule],
) -> List[FirewallRule]:
    """Convert Firewall Rules into a Python class suitable for comparison.

    NOTE: We only check "ALLOW" rules & do not check priority.
    """
    return [
        FirewallRule(
            direction=rule.direction,
            protocol=Protocol.from_val(l4config.ip_protocol),
            ports=l4config.ports,
            network=ipaddress.ip_network(addr),
        )
        for rule in rules
        if rule.action == "allow"
        for addr in rule.match.src_ip_ranges
        for l4config in (
            rule.match.layer4_configs
            or [
                compute_v1.types.compute.FirewallPolicyRuleMatcherLayer4Config(
                    ip_protocol="all"
                )
            ]
        )
    ]


def verify_filestore(
    factory: GoogleCloudClientFactory,
    file_store_instance_name: str,
    vpc_name: Optional[str],
    cloud_region: str,
    logger: GCPLogger,
    strict: bool = False,
) -> bool:
    """Verify Filestore exists & that it is connected to the correct VPC.

    TODO: Warn about Filestore size if it is 'too small'."""
    client = factory.filestore_v1.CloudFilestoreManagerClient()
    try:
        file_store = client.get_instance(name=file_store_instance_name)
    except NotFound:
        logger.log_resource_not_found_error("Filestore", file_store_instance_name)
        return False

    file_store_match = re.match(_FILESTORE_NAME_REGEX_PATTERN, file_store.name)
    if file_store_match:
        filestore_location = file_store_match.group("location")
        # Filestore location can be region or zone.
        if cloud_region not in filestore_location:
            logger.internal.warning(
                f"Filestore is in {filestore_location}, but this cloud is being set up in {cloud_region}. This can result in cross-region network egress charges, refer to https://cloud.google.com/vpc/network-pricing#egress-within-gcp."
            )
            if strict:
                return False
    else:
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_FILESTORE,
            CloudSetupError.FILESTORE_NAME_MALFORMED,
        )
        logger.internal.error(
            f"Failed to get filestore location, since filestore name doesn't match format {_FILESTORE_NAME_REGEX_PATTERN}!"
        )
        return False

    file_store_networks = [v.network for v in file_store.networks]
    if not any(vpc_name == network.split("/")[-1] for network in file_store_networks):
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_FILESTORE,
            CloudSetupError.FILESTORE_NOT_CONNECTED_TO_VPC,
        )
        logger.internal.error(
            f"Filestore is not connected to {vpc_name}, but to {file_store_networks}. "
            f"This cannot be edited on an existing Filestore instance. Please recreate the filestore and connect it to {vpc_name}."
        )
        return False

    return True


def verify_cloud_storage(  # noqa: PLR0911, PLR0912, PLR0913
    factory: GoogleCloudClientFactory,
    bucket_name: Optional[str],
    controlplane_service_account: Optional[str],
    dataplane_service_account: Optional[str],
    project_id: str,
    cloud_region: str,
    logger: GCPLogger,
    strict: bool = False,
):
    """Verify that the Google Cloud Storage Bucket exists & raises warnings about improper configurations."""
    bucket_client = factory.storage.Client(project_id)
    try:
        bucket = bucket_client.get_bucket(bucket_name)
    except NotFound:
        logger.log_resource_not_found_error("Google Cloud Storage Bucket", bucket_name)
        return False

    if not bucket.iam_configuration.uniform_bucket_level_access_enabled:
        logger.internal.warning(
            f"Bucket {bucket_name}, does not have Uniform Bucket Access enabled, "
            "this impedes Anyscale's ability to verify bucket access."
        )

    public_access_protection = bucket.iam_configuration.public_access_prevention
    if public_access_protection not in (
        PUBLIC_ACCESS_PREVENTION_INHERITED,
        PUBLIC_ACCESS_PREVENTION_ENFORCED,
    ):
        logger.internal.warning(
            f"Bucket {bucket_name} has public access prevention set to {public_access_protection}, not enforced or inherited."
        )
        if strict:
            return False

    correct_region, bucket_region = _check_bucket_region(bucket, cloud_region)
    if not correct_region:
        logger.internal.warning(
            f"Bucket {bucket_name} is in region {bucket_region}, but this cloud is being set up in {cloud_region}."
            "This can result in degraded cluster launch & logging performance."
        )
        if strict:
            return False

    has_correct_cors, _cors_reason = check_gcp_cors_rules(bucket.cors or [])
    if not has_correct_cors:
        logger.internal.warning(
            f"Bucket {bucket_name} does not have the correct CORS rule for Anyscale. This is safe to ignore if you are not using Anyscale UI.\n"
            "If you are using the UI, please create the correct CORS rule for Anyscale according to https://docs.anyscale.com/cloud-deployment/gcp/deploy-cloud?cloud-deployment=custom#4-create-an-anyscale-cloud"
        )
        if strict:
            return False

    iam_policy = bucket.get_iam_policy()
    iam_bindings = binding_from_dictionary(iam_policy.bindings)

    permission_warning = (
        "The {location} Service Account {email} requires the following permissions on Bucket {bucket} to operate correctly:\n"
        "* storage.buckets.get\n* storage.objects.[ get | list | create ]\n* storage.multipartUploads.[ abort | create | listParts ]\n"
        "We suggest granting the predefined roles of `roles/storage.legacyBucketReader` and `roles/storage.objectAdmin`."
    )

    if not _verify_service_account_on_bucket(
        f"serviceAccount:{controlplane_service_account}", iam_bindings
    ):
        logger.confirm_missing_permission(
            permission_warning.format(
                location="Anyscale access",
                email=controlplane_service_account,
                bucket=bucket_name,
            )
        )
        if strict:
            return False

    if not _verify_service_account_on_bucket(
        f"serviceAccount:{dataplane_service_account}", iam_bindings
    ):
        logger.confirm_missing_permission(
            permission_warning.format(
                location="Dataplane",
                email=dataplane_service_account,
                bucket=bucket_name,
            )
        )
        if strict:
            return False

    return True


def _verify_service_account_on_bucket(
    service_account: str, iam_bindings: List[Binding]
) -> bool:
    """Verifies that the given service account has roles that ensure the following list of permissions:
    * storage.buckets.get: Get bucket info
    * storage.objects.[ get | list | create ]
    * storage.multipartUploads.[ abort | create | listParts ]
    """
    if check_policy_bindings(iam_bindings, service_account, {"roles/storage.admin"},):
        return True
    return any(
        check_required_policy_bindings(iam_bindings, service_account, combination)
        for combination in [
            {
                # Best Permissions (legacy for `buckets.get`)
                "roles/storage.legacyBucketReader",
                "roles/storage.objectViewer",
                "roles/storage.objectCreator",
            },
            {
                # Most Brief permissions (legacy for `buckets.get`)
                "roles/storage.objectAdmin",
                "roles/storage.legacyBucketReader",
            },
            {
                # Optimal Legacy Roles
                "roles/storage.legacyBucketWriter",
                "roles/storage.legacyObjectReader",
            },
            {
                # Legacy Roles (extra object permissions)
                "roles/storage.legacyBucketWriter",
                "roles/storage.legacyObjectOwner",
            },
            {
                # Legacy Roles (extra bucket permissions)
                "roles/storage.legacyBucketOwner",
                "roles/storage.legacyObjectReader",
            },
            {
                # Legacy Roles (extra bucket/object permissions)
                "roles/storage.legacyBucketOwner",
                "roles/storage.legacyObjectOwner",
            },
        ]
    )


def _check_bucket_region(bucket: Bucket, region: str) -> Tuple[bool, str]:
    if bucket.location_type == "dual-region":
        return region.upper() in bucket.data_locations, ",".join(bucket.data_locations)
    elif bucket.location_type == "region":
        return region.upper() == bucket.location, bucket.location

    # Bucket is `multi-region`, so check if the location (`EU`, `ASIA`, `US`)
    # is the 'prefix' of the region.
    return region.upper().startswith(bucket.location), bucket.location


def verify_memorystore(  # noqa: PLR0911
    factory: GoogleCloudClientFactory,
    redis_instance_name: str,
    cloud_vpc_name: Optional[str],
    logger: GCPLogger,
    strict: bool = False,
):
    """Verify that the Google Memorystore exists & raises warnings about improper configurations."""
    client = factory.redis_v1.CloudRedisClient()
    try:
        redis_instance = client.get_instance(name=redis_instance_name)
    except (NotFound, PermissionDenied):
        logger.log_resource_not_found_error("Memorystore", redis_instance_name)
        return False

    # Verify memorystore is in the same VPC as the cloud
    redis_vpc_name = redis_instance.authorized_network.split("/")[-1]
    if cloud_vpc_name != redis_vpc_name:
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_MEMORYSTORE,
            CloudSetupError.MEMORYSTORE_NOT_CONNECTED_TO_VPC,
        )
        logger.internal.error(
            f"Memorystore is not connected to {cloud_vpc_name}, but to {redis_vpc_name}. "
            f"This cannot be edited on an existing Memorystore instance. Please recreate the Memorystore and connect it to {cloud_vpc_name}."
        )
        return False

    # Verify memorystore is the standard tier
    if redis_instance.tier.name != "STANDARD_HA":
        logger.internal.warning(
            "Memorystore is not in the standard tier. This can result in degraded performance and availability. Note that this cannot be changed on an existing Memorystore instance."
        )
        if strict:
            return False

    # Verify that memorystore instance has the maxmemory-policy set to allkeys-lru
    if redis_instance.redis_configs.get("maxmemory-policy") != "allkeys-lru":
        logger.internal.warning(
            "Memorystore does not have the maxmemory-policy set to allkeys-lru. This can result in degraded performance and availability."
        )
        if strict:
            return False

    # Verify that the memorystore instance has the read replica mode turned on
    if redis_instance.read_replicas_mode.name != "READ_REPLICAS_ENABLED":
        logger.internal.error(
            "Memorystore does not have the read replica mode turned on. This is required for ray head node HA."
        )
        logger.internal.log_resource_error(
            CloudAnalyticsEventCloudResource.GCP_MEMORYSTORE,
            CloudSetupError.MEMORYSTORE_READ_REPLICAS_DISABLED,
        )
        return False

    # Verify that the memorystore has tls disabled
    if redis_instance.transit_encryption_mode.name != "DISABLED":
        logger.internal.error(
            "Memorystore has TLS enabled. Please create a new memorystore instance with TLS disabled."
        )
        return False

    return True



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class DescribeSystemWorkloadResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster_id': 'str',
        'workload_names': 'list[SystemWorkloadName]',
        'workload_service_url': 'str',
        'workload_service_url_auth': 'str',
        'status': 'ClusterState',
        'is_enabled': 'bool'
    }

    attribute_map = {
        'cluster_id': 'cluster_id',
        'workload_names': 'workload_names',
        'workload_service_url': 'workload_service_url',
        'workload_service_url_auth': 'workload_service_url_auth',
        'status': 'status',
        'is_enabled': 'is_enabled'
    }

    def __init__(self, cluster_id=None, workload_names=None, workload_service_url=None, workload_service_url_auth=None, status=None, is_enabled=None, local_vars_configuration=None):  # noqa: E501
        """DescribeSystemWorkloadResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._cluster_id = None
        self._workload_names = None
        self._workload_service_url = None
        self._workload_service_url_auth = None
        self._status = None
        self._is_enabled = None
        self.discriminator = None

        if cluster_id is not None:
            self.cluster_id = cluster_id
        self.workload_names = workload_names
        if workload_service_url is not None:
            self.workload_service_url = workload_service_url
        if workload_service_url_auth is not None:
            self.workload_service_url_auth = workload_service_url_auth
        if status is not None:
            self.status = status
        self.is_enabled = is_enabled

    @property
    def cluster_id(self):
        """Gets the cluster_id of this DescribeSystemWorkloadResponse.  # noqa: E501


        :return: The cluster_id of this DescribeSystemWorkloadResponse.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this DescribeSystemWorkloadResponse.


        :param cluster_id: The cluster_id of this DescribeSystemWorkloadResponse.  # noqa: E501
        :type: str
        """

        self._cluster_id = cluster_id

    @property
    def workload_names(self):
        """Gets the workload_names of this DescribeSystemWorkloadResponse.  # noqa: E501


        :return: The workload_names of this DescribeSystemWorkloadResponse.  # noqa: E501
        :rtype: list[SystemWorkloadName]
        """
        return self._workload_names

    @workload_names.setter
    def workload_names(self, workload_names):
        """Sets the workload_names of this DescribeSystemWorkloadResponse.


        :param workload_names: The workload_names of this DescribeSystemWorkloadResponse.  # noqa: E501
        :type: list[SystemWorkloadName]
        """
        if self.local_vars_configuration.client_side_validation and workload_names is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_names`, must not be `None`")  # noqa: E501

        self._workload_names = workload_names

    @property
    def workload_service_url(self):
        """Gets the workload_service_url of this DescribeSystemWorkloadResponse.  # noqa: E501


        :return: The workload_service_url of this DescribeSystemWorkloadResponse.  # noqa: E501
        :rtype: str
        """
        return self._workload_service_url

    @workload_service_url.setter
    def workload_service_url(self, workload_service_url):
        """Sets the workload_service_url of this DescribeSystemWorkloadResponse.


        :param workload_service_url: The workload_service_url of this DescribeSystemWorkloadResponse.  # noqa: E501
        :type: str
        """

        self._workload_service_url = workload_service_url

    @property
    def workload_service_url_auth(self):
        """Gets the workload_service_url_auth of this DescribeSystemWorkloadResponse.  # noqa: E501


        :return: The workload_service_url_auth of this DescribeSystemWorkloadResponse.  # noqa: E501
        :rtype: str
        """
        return self._workload_service_url_auth

    @workload_service_url_auth.setter
    def workload_service_url_auth(self, workload_service_url_auth):
        """Sets the workload_service_url_auth of this DescribeSystemWorkloadResponse.


        :param workload_service_url_auth: The workload_service_url_auth of this DescribeSystemWorkloadResponse.  # noqa: E501
        :type: str
        """

        self._workload_service_url_auth = workload_service_url_auth

    @property
    def status(self):
        """Gets the status of this DescribeSystemWorkloadResponse.  # noqa: E501


        :return: The status of this DescribeSystemWorkloadResponse.  # noqa: E501
        :rtype: ClusterState
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this DescribeSystemWorkloadResponse.


        :param status: The status of this DescribeSystemWorkloadResponse.  # noqa: E501
        :type: ClusterState
        """

        self._status = status

    @property
    def is_enabled(self):
        """Gets the is_enabled of this DescribeSystemWorkloadResponse.  # noqa: E501


        :return: The is_enabled of this DescribeSystemWorkloadResponse.  # noqa: E501
        :rtype: bool
        """
        return self._is_enabled

    @is_enabled.setter
    def is_enabled(self, is_enabled):
        """Sets the is_enabled of this DescribeSystemWorkloadResponse.


        :param is_enabled: The is_enabled of this DescribeSystemWorkloadResponse.  # noqa: E501
        :type: bool
        """
        if self.local_vars_configuration.client_side_validation and is_enabled is None:  # noqa: E501
            raise ValueError("Invalid value for `is_enabled`, must not be `None`")  # noqa: E501

        self._is_enabled = is_enabled

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DescribeSystemWorkloadResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DescribeSystemWorkloadResponse):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TaskTableRow(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'attempt_number': 'int',
        'total_attempts': 'int',
        'job_id': 'str',
        'function_name': 'str',
        'task_type': 'TaskType',
        'current_state': 'TaskState',
        'error_message': 'str',
        'start_time_ns': 'int',
        'start_running_time_ns': 'int',
        'end_time_ns': 'int',
        'required_resources': 'str',
        'runtime_env': 'str',
        'node_id': 'str',
        'worker_id': 'str',
        'worker_pid': 'str',
        'parent_task_id': 'str',
        'ray_session_name': 'str',
        'exception_type': 'str',
        'data_operator_id': 'str'
    }

    attribute_map = {
        'id': 'id',
        'attempt_number': 'attempt_number',
        'total_attempts': 'total_attempts',
        'job_id': 'job_id',
        'function_name': 'function_name',
        'task_type': 'task_type',
        'current_state': 'current_state',
        'error_message': 'error_message',
        'start_time_ns': 'start_time_ns',
        'start_running_time_ns': 'start_running_time_ns',
        'end_time_ns': 'end_time_ns',
        'required_resources': 'required_resources',
        'runtime_env': 'runtime_env',
        'node_id': 'node_id',
        'worker_id': 'worker_id',
        'worker_pid': 'worker_pid',
        'parent_task_id': 'parent_task_id',
        'ray_session_name': 'ray_session_name',
        'exception_type': 'exception_type',
        'data_operator_id': 'data_operator_id'
    }

    def __init__(self, id=None, attempt_number=None, total_attempts=None, job_id=None, function_name=None, task_type=None, current_state=None, error_message=None, start_time_ns=None, start_running_time_ns=None, end_time_ns=None, required_resources=None, runtime_env=None, node_id=None, worker_id=None, worker_pid=None, parent_task_id=None, ray_session_name=None, exception_type=None, data_operator_id=None, local_vars_configuration=None):  # noqa: E501
        """TaskTableRow - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._attempt_number = None
        self._total_attempts = None
        self._job_id = None
        self._function_name = None
        self._task_type = None
        self._current_state = None
        self._error_message = None
        self._start_time_ns = None
        self._start_running_time_ns = None
        self._end_time_ns = None
        self._required_resources = None
        self._runtime_env = None
        self._node_id = None
        self._worker_id = None
        self._worker_pid = None
        self._parent_task_id = None
        self._ray_session_name = None
        self._exception_type = None
        self._data_operator_id = None
        self.discriminator = None

        self.id = id
        self.attempt_number = attempt_number
        if total_attempts is not None:
            self.total_attempts = total_attempts
        self.job_id = job_id
        self.function_name = function_name
        self.task_type = task_type
        if current_state is not None:
            self.current_state = current_state
        if error_message is not None:
            self.error_message = error_message
        self.start_time_ns = start_time_ns
        self.start_running_time_ns = start_running_time_ns
        if end_time_ns is not None:
            self.end_time_ns = end_time_ns
        if required_resources is not None:
            self.required_resources = required_resources
        if runtime_env is not None:
            self.runtime_env = runtime_env
        if node_id is not None:
            self.node_id = node_id
        if worker_id is not None:
            self.worker_id = worker_id
        if worker_pid is not None:
            self.worker_pid = worker_pid
        if parent_task_id is not None:
            self.parent_task_id = parent_task_id
        self.ray_session_name = ray_session_name
        if exception_type is not None:
            self.exception_type = exception_type
        if data_operator_id is not None:
            self.data_operator_id = data_operator_id

    @property
    def id(self):
        """Gets the id of this TaskTableRow.  # noqa: E501


        :return: The id of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this TaskTableRow.


        :param id: The id of this TaskTableRow.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def attempt_number(self):
        """Gets the attempt_number of this TaskTableRow.  # noqa: E501


        :return: The attempt_number of this TaskTableRow.  # noqa: E501
        :rtype: int
        """
        return self._attempt_number

    @attempt_number.setter
    def attempt_number(self, attempt_number):
        """Sets the attempt_number of this TaskTableRow.


        :param attempt_number: The attempt_number of this TaskTableRow.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and attempt_number is None:  # noqa: E501
            raise ValueError("Invalid value for `attempt_number`, must not be `None`")  # noqa: E501

        self._attempt_number = attempt_number

    @property
    def total_attempts(self):
        """Gets the total_attempts of this TaskTableRow.  # noqa: E501


        :return: The total_attempts of this TaskTableRow.  # noqa: E501
        :rtype: int
        """
        return self._total_attempts

    @total_attempts.setter
    def total_attempts(self, total_attempts):
        """Sets the total_attempts of this TaskTableRow.


        :param total_attempts: The total_attempts of this TaskTableRow.  # noqa: E501
        :type: int
        """

        self._total_attempts = total_attempts

    @property
    def job_id(self):
        """Gets the job_id of this TaskTableRow.  # noqa: E501


        :return: The job_id of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this TaskTableRow.


        :param job_id: The job_id of this TaskTableRow.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and job_id is None:  # noqa: E501
            raise ValueError("Invalid value for `job_id`, must not be `None`")  # noqa: E501

        self._job_id = job_id

    @property
    def function_name(self):
        """Gets the function_name of this TaskTableRow.  # noqa: E501


        :return: The function_name of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._function_name

    @function_name.setter
    def function_name(self, function_name):
        """Sets the function_name of this TaskTableRow.


        :param function_name: The function_name of this TaskTableRow.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and function_name is None:  # noqa: E501
            raise ValueError("Invalid value for `function_name`, must not be `None`")  # noqa: E501

        self._function_name = function_name

    @property
    def task_type(self):
        """Gets the task_type of this TaskTableRow.  # noqa: E501


        :return: The task_type of this TaskTableRow.  # noqa: E501
        :rtype: TaskType
        """
        return self._task_type

    @task_type.setter
    def task_type(self, task_type):
        """Sets the task_type of this TaskTableRow.


        :param task_type: The task_type of this TaskTableRow.  # noqa: E501
        :type: TaskType
        """
        if self.local_vars_configuration.client_side_validation and task_type is None:  # noqa: E501
            raise ValueError("Invalid value for `task_type`, must not be `None`")  # noqa: E501

        self._task_type = task_type

    @property
    def current_state(self):
        """Gets the current_state of this TaskTableRow.  # noqa: E501


        :return: The current_state of this TaskTableRow.  # noqa: E501
        :rtype: TaskState
        """
        return self._current_state

    @current_state.setter
    def current_state(self, current_state):
        """Sets the current_state of this TaskTableRow.


        :param current_state: The current_state of this TaskTableRow.  # noqa: E501
        :type: TaskState
        """

        self._current_state = current_state

    @property
    def error_message(self):
        """Gets the error_message of this TaskTableRow.  # noqa: E501


        :return: The error_message of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._error_message

    @error_message.setter
    def error_message(self, error_message):
        """Sets the error_message of this TaskTableRow.


        :param error_message: The error_message of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._error_message = error_message

    @property
    def start_time_ns(self):
        """Gets the start_time_ns of this TaskTableRow.  # noqa: E501


        :return: The start_time_ns of this TaskTableRow.  # noqa: E501
        :rtype: int
        """
        return self._start_time_ns

    @start_time_ns.setter
    def start_time_ns(self, start_time_ns):
        """Sets the start_time_ns of this TaskTableRow.


        :param start_time_ns: The start_time_ns of this TaskTableRow.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and start_time_ns is None:  # noqa: E501
            raise ValueError("Invalid value for `start_time_ns`, must not be `None`")  # noqa: E501

        self._start_time_ns = start_time_ns

    @property
    def start_running_time_ns(self):
        """Gets the start_running_time_ns of this TaskTableRow.  # noqa: E501


        :return: The start_running_time_ns of this TaskTableRow.  # noqa: E501
        :rtype: int
        """
        return self._start_running_time_ns

    @start_running_time_ns.setter
    def start_running_time_ns(self, start_running_time_ns):
        """Sets the start_running_time_ns of this TaskTableRow.


        :param start_running_time_ns: The start_running_time_ns of this TaskTableRow.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and start_running_time_ns is None:  # noqa: E501
            raise ValueError("Invalid value for `start_running_time_ns`, must not be `None`")  # noqa: E501

        self._start_running_time_ns = start_running_time_ns

    @property
    def end_time_ns(self):
        """Gets the end_time_ns of this TaskTableRow.  # noqa: E501


        :return: The end_time_ns of this TaskTableRow.  # noqa: E501
        :rtype: int
        """
        return self._end_time_ns

    @end_time_ns.setter
    def end_time_ns(self, end_time_ns):
        """Sets the end_time_ns of this TaskTableRow.


        :param end_time_ns: The end_time_ns of this TaskTableRow.  # noqa: E501
        :type: int
        """

        self._end_time_ns = end_time_ns

    @property
    def required_resources(self):
        """Gets the required_resources of this TaskTableRow.  # noqa: E501


        :return: The required_resources of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._required_resources

    @required_resources.setter
    def required_resources(self, required_resources):
        """Sets the required_resources of this TaskTableRow.


        :param required_resources: The required_resources of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._required_resources = required_resources

    @property
    def runtime_env(self):
        """Gets the runtime_env of this TaskTableRow.  # noqa: E501


        :return: The runtime_env of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._runtime_env

    @runtime_env.setter
    def runtime_env(self, runtime_env):
        """Sets the runtime_env of this TaskTableRow.


        :param runtime_env: The runtime_env of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._runtime_env = runtime_env

    @property
    def node_id(self):
        """Gets the node_id of this TaskTableRow.  # noqa: E501


        :return: The node_id of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._node_id

    @node_id.setter
    def node_id(self, node_id):
        """Sets the node_id of this TaskTableRow.


        :param node_id: The node_id of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._node_id = node_id

    @property
    def worker_id(self):
        """Gets the worker_id of this TaskTableRow.  # noqa: E501


        :return: The worker_id of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._worker_id

    @worker_id.setter
    def worker_id(self, worker_id):
        """Sets the worker_id of this TaskTableRow.


        :param worker_id: The worker_id of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._worker_id = worker_id

    @property
    def worker_pid(self):
        """Gets the worker_pid of this TaskTableRow.  # noqa: E501


        :return: The worker_pid of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._worker_pid

    @worker_pid.setter
    def worker_pid(self, worker_pid):
        """Sets the worker_pid of this TaskTableRow.


        :param worker_pid: The worker_pid of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._worker_pid = worker_pid

    @property
    def parent_task_id(self):
        """Gets the parent_task_id of this TaskTableRow.  # noqa: E501


        :return: The parent_task_id of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._parent_task_id

    @parent_task_id.setter
    def parent_task_id(self, parent_task_id):
        """Sets the parent_task_id of this TaskTableRow.


        :param parent_task_id: The parent_task_id of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._parent_task_id = parent_task_id

    @property
    def ray_session_name(self):
        """Gets the ray_session_name of this TaskTableRow.  # noqa: E501


        :return: The ray_session_name of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._ray_session_name

    @ray_session_name.setter
    def ray_session_name(self, ray_session_name):
        """Sets the ray_session_name of this TaskTableRow.


        :param ray_session_name: The ray_session_name of this TaskTableRow.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and ray_session_name is None:  # noqa: E501
            raise ValueError("Invalid value for `ray_session_name`, must not be `None`")  # noqa: E501

        self._ray_session_name = ray_session_name

    @property
    def exception_type(self):
        """Gets the exception_type of this TaskTableRow.  # noqa: E501


        :return: The exception_type of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._exception_type

    @exception_type.setter
    def exception_type(self, exception_type):
        """Sets the exception_type of this TaskTableRow.


        :param exception_type: The exception_type of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._exception_type = exception_type

    @property
    def data_operator_id(self):
        """Gets the data_operator_id of this TaskTableRow.  # noqa: E501


        :return: The data_operator_id of this TaskTableRow.  # noqa: E501
        :rtype: str
        """
        return self._data_operator_id

    @data_operator_id.setter
    def data_operator_id(self, data_operator_id):
        """Sets the data_operator_id of this TaskTableRow.


        :param data_operator_id: The data_operator_id of this TaskTableRow.  # noqa: E501
        :type: str
        """

        self._data_operator_id = data_operator_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TaskTableRow):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TaskTableRow):
            return True

        return self.to_dict() != other.to_dict()



import re
from typing import Dict, List, Tuple


ANYSCALE_SDK_INTRO = """\
The AnyscaleSDK class must be constructed in order to make calls to the SDK. This class allows you to create an authenticated client in which to use the SDK.

| Param | Type | Description |
| :--- | :--- | :--- |
| `auth_token` | Optional String | Authentication token used to verify you have permissions to access Anyscale. If not provided, permissions default to the credentials set for your current user. Credentials can be set by following the instructions on this page: https://console.anyscale.com/credentials |

**Example**
```python
from anyscale import AnyscaleSDK

sdk = AnyscaleSDK()
```
"""


def _build_model_to_module_mapping() -> Dict[str, str]:
    """Build mapping from model name (lowercase) to module filename.

    This dynamically discovers the mapping from ALL_MODULES configuration,
    eliminating the need for hardcoded constants.
    """
    # Import here to avoid circular imports
    from anyscale._private.docgen.__main__ import (  # noqa: PLC0415 - codex_reason("gpt5.2", "avoid circular import at module load")
        ALL_MODULES,
    )

    model_name_to_file = {}

    for module in ALL_MODULES:
        # Current models (Python types)
        for model_type in module.models:
            model_name_to_file[model_type.__name__.lower()] = module.filename

        # Legacy models (strings)
        if module.legacy_sdk_models:
            for model_name in module.legacy_sdk_models:
                model_name_to_file[model_name.lower()] = module.filename

    return model_name_to_file


def _transform_legacy_links(text: str, for_legacy_file: bool = True) -> str:
    """Transform legacy model links to include proper cross-module references.

    Args:
        text: The text containing links to transform
        for_legacy_file: If True, generate links for legacy files (in legacy/ subfolder).
                        If False, generate links for main files with -legacy anchors.
    """
    # Build the mapping dynamically
    model_mapping = _build_model_to_module_mapping()

    def replace_link(match):
        model_name = match.group(1)
        if model_name in model_mapping:
            module_file = model_mapping[model_name]
            if for_legacy_file:
                # In legacy files, link to other legacy files without -legacy suffix
                return f"({module_file}#{model_name})"
            # In main files, link with -legacy suffix
            return f"({module_file}#{model_name}-legacy)"
        # Fallback for unmapped models (stay in same file)
        if for_legacy_file:
            return f"(#{model_name})"
        return f"(#{model_name}-legacy)"

    # Transform links from (#modelname) to proper cross-module references
    text = re.sub(r"\(#([a-z]+)\)", replace_link, text)

    # Transform workspace command references
    if for_legacy_file:
        # In legacy files, point to legacy workspace file
        text = re.sub(
            r"\(#anyscale-workspace_v2-([a-z]+)\)",
            r"(workspaces.md#anyscale-workspace_v2-\1)",
            text,
        )
    else:
        # In main files, point to main workspace file
        text = re.sub(
            r"\(#anyscale-workspace_v2-([a-z]+)\)",
            r"(workspaces.md#anyscale-workspace_v2-\1)",
            text,
        )

    return text


class LegacySDK:
    def __init__(self, name: str, docstring: str):
        self.name = name
        self.docstring = docstring

    @classmethod
    def from_md(cls, md: str) -> "LegacySDK":
        """
        Convert a blob of markdown into a LegacySDK object.
        """
        name = ""
        docstring = ""

        for line in md.split("\n"):
            if line.startswith("### "):
                name = line[4:]
            else:
                # Transform ./models.md links and local links
                transformed = _transform_legacy_links(
                    re.sub(r"\(./models\.md#([a-z]+)\)", r"(#\1-legacy)", line),
                    for_legacy_file=False,
                )
                docstring += transformed + "\n"

        return cls(name=name, docstring=docstring.strip())


class LegacyModel:
    def __init__(self, name: str, docstring: str):
        self.name = name
        self.docstring = docstring

    @classmethod
    def from_md(cls, md: str) -> "LegacyModel":
        """
        Convert a blob of markdown into a LegacyModel object.
        """
        name = ""
        docstring = ""

        for line in md.split("\n"):
            if line.startswith("## "):
                name = line[3:]
            else:
                docstring += _transform_legacy_links(line, for_legacy_file=False) + "\n"

        return cls(name=name, docstring=docstring.strip())


def _chunk(md_file: str, start: str, ends: List[str]) -> List[str]:
    """
    Split a markdown file into chunks based on the header.
    """
    chunks = []
    with open(md_file) as f:
        line = f.readline()
        while line:
            if not line.startswith(start):
                line = f.readline()
                continue
            chunk = line
            line = f.readline()
            while line and not any(line.startswith(end) for end in ends):
                chunk += line
                line = f.readline()

            chunks.append(chunk)

    return chunks


def parse_legacy_sdks(
    api_md_file: str, model_md_file: str
) -> Tuple[List[LegacySDK], List[LegacyModel]]:
    """
    Parse the legacy SDK markdown files into a list of LegacySDK objects.
    """
    legacy_sdks = [
        LegacySDK.from_md(chunk)
        for chunk in _chunk(api_md_file, "### ", ["### ", "## ", "# "])
    ]
    legacy_models = [
        LegacyModel.from_md(chunk)
        for chunk in _chunk(model_md_file, "## ", ["## ", "# "])
    ]

    return legacy_sdks, legacy_models



from typing import Dict, List, Optional, Union

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.service._private.service_sdk import PrivateServiceSDK
from anyscale.service.commands import (
    _ARCHIVE_ARG_DOCSTRINGS,
    _ARCHIVE_EXAMPLE,
    _controller_logs as _controller_logs,
    _CONTROLLER_LOGS_ARG_DOCSTRINGS,
    _CONTROLLER_LOGS_EXAMPLE,
    _DELETE_ARG_DOCSTRINGS,
    _DELETE_EXAMPLE,
    _DEPLOY_ARG_DOCSTRINGS,
    _DEPLOY_EXAMPLE,
    _LIST_ARG_DOCSTRINGS,
    _LIST_EXAMPLE,
    _ROLLBACK_ARG_DOCSTRINGS,
    _ROLLBACK_EXAMPLE,
    _STATUS_ARG_DOCSTRINGS,
    _STATUS_EXAMPLE,
    _TAGS_ADD_ARG_DOCSTRINGS,
    _TAGS_ADD_EXAMPLE,
    _TAGS_LIST_ARG_DOCSTRINGS,
    _TAGS_LIST_EXAMPLE,
    _TAGS_REMOVE_ARG_DOCSTRINGS,
    _TAGS_REMOVE_EXAMPLE,
    _TERMINATE_ARG_DOCSTRINGS,
    _TERMINATE_EXAMPLE,
    _WAIT_ARG_DOCSTRINGS,
    _WAIT_EXAMPLE,
    add_tags as add_tags,
    archive as archive,
    delete as delete,
    deploy as deploy,
    list as list,  # noqa: A004 - claude_comment("claude-opus-4-5", "SDK public API re-export")
    list_tags as list_tags,
    remove_tags as remove_tags,
    rollback as rollback,
    status as status,
    terminate as terminate,
    wait as wait,
)
from anyscale.service.models import (
    ServiceConfig,
    ServiceLogMode,
    ServiceSortField,
    ServiceSortOrder,
    ServiceState,
    ServiceStatus,
)


class ServiceSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateServiceSDK(client=client, logger=logger, timer=timer)

    @sdk_docs(
        doc_py_example=_DEPLOY_EXAMPLE, arg_docstrings=_DEPLOY_ARG_DOCSTRINGS,
    )
    def deploy(  # noqa: F811
        self,
        configs: Union[ServiceConfig, List[ServiceConfig]],
        *,
        in_place: bool = False,
        canary_percent: Optional[int] = None,
        max_surge_percent: Optional[int] = None,
        versions: Optional[str] = None,
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        """Deploy a service.

        If no service with the provided name is running, one will be created, else the existing service will be updated.

        This command is asynchronous, so it always returns immediately.

        Returns the id of the deployed service.
        """
        return self._private_sdk.deploy(
            configs=configs,
            in_place=in_place,
            canary_percent=canary_percent,
            max_surge_percent=max_surge_percent,
            versions=versions,
            name=name,
            cloud=cloud,
            project=project,
        )

    @sdk_docs(
        doc_py_example=_ROLLBACK_EXAMPLE, arg_docstrings=_ROLLBACK_ARG_DOCSTRINGS,
    )
    def rollback(  # noqa: F811
        self,
        name: Optional[str] = None,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        max_surge_percent: Optional[int] = None,
    ) -> str:
        """Rollback to the primary version of the service.

        This command can only be used when there is an active rollout in progress. The
        rollout will be cancelled and the service will revert to the primary version.

        This command is asynchronous, so it always returns immediately.

        Returns the id of the rolled back service.
        """
        return self._private_sdk.rollback(
            name=name, max_surge_percent=max_surge_percent, cloud=cloud, project=project
        )

    @sdk_docs(
        doc_py_example=_TERMINATE_EXAMPLE, arg_docstrings=_TERMINATE_ARG_DOCSTRINGS,
    )
    def terminate(  # noqa: F811
        self,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        """Terminate a service.

        This command is asynchronous, so it always returns immediately.

        Returns the id of the terminated service.
        """
        return self._private_sdk.terminate(
            id=id, name=name, cloud=cloud, project=project
        )

    @sdk_docs(
        doc_py_example=_ARCHIVE_EXAMPLE, arg_docstrings=_ARCHIVE_ARG_DOCSTRINGS,
    )
    def archive(  # noqa: F811
        self,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        """Archive a service.

        This command is asynchronous, so it always returns immediately.

        Returns the ID of the archived service.
        """
        return self._private_sdk.archive(id=id, name=name, cloud=cloud, project=project)

    @sdk_docs(
        doc_py_example=_DELETE_EXAMPLE, arg_docstrings=_DELETE_ARG_DOCSTRINGS,
    )
    def delete(  # noqa: F811
        self,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ):
        """Delete a service.

        This command is asynchronous, so it always returns immediately.
        """
        return self._private_sdk.delete(id=id, name=name, cloud=cloud, project=project)

    @sdk_docs(
        doc_py_example=_LIST_EXAMPLE, arg_docstrings=_LIST_ARG_DOCSTRINGS,
    )
    def list(  # noqa: A001, F811, PLR0913
        self,
        *,
        # Single-item lookup
        service_id: Optional[str] = None,
        # Filters
        name: Optional[str] = None,
        state_filter: Optional[Union[List[ServiceState], List[str]]] = None,
        tags_filter: Optional[Dict[str, List[str]]] = None,
        creator_id: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        include_archived: bool = False,
        # Paging
        max_items: Optional[int] = None,
        page_size: Optional[int] = None,
        # Sorting
        sort_field: Optional[Union[str, ServiceSortField]] = None,
        sort_order: Optional[Union[str, ServiceSortOrder]] = None,
    ) -> ResultIterator[ServiceStatus]:
        """List services.

        Returns
        -------
        ResultIterator[ServiceStatus]
            An iterator yielding ServiceStatus objects. Fetches pages
            lazily as needed. Use a `for` loop to iterate or `list()`
            to consume all at once.
        """
        return self._private_sdk.list(
            service_id=service_id,
            name=name,
            state_filter=state_filter,
            tags_filter=tags_filter,
            creator_id=creator_id,
            cloud=cloud,
            project=project,
            include_archived=include_archived,
            max_items=max_items,
            page_size=page_size,
            sort_field=sort_field,
            sort_order=sort_order,
        )

    @sdk_docs(
        doc_py_example=_STATUS_EXAMPLE, arg_docstrings=_STATUS_ARG_DOCSTRINGS,
    )
    def status(  # noqa: F811
        self, name: str, *, cloud: Optional[str] = None, project: Optional[str] = None
    ) -> ServiceStatus:
        """Get the status of a service."""
        return self._private_sdk.status(name=name, cloud=cloud, project=project)

    @sdk_docs(
        doc_py_example=_WAIT_EXAMPLE, arg_docstrings=_WAIT_ARG_DOCSTRINGS,
    )
    def wait(  # noqa: F811
        self,
        name: str,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        state: Union[str, ServiceState] = ServiceState.RUNNING,
        timeout_s: float = 600,
        _interval_s: float = 5,
    ):
        """Wait for a service to reach a target state."""
        return self._private_sdk.wait(
            name=name,
            cloud=cloud,
            project=project,
            state=ServiceState(state),
            timeout_s=timeout_s,
            interval_s=_interval_s,
        )

    @sdk_docs(
        doc_py_example=_CONTROLLER_LOGS_EXAMPLE,
        arg_docstrings=_CONTROLLER_LOGS_ARG_DOCSTRINGS,
    )
    def controller_logs(  # noqa: F811
        self,
        name: str,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        canary: bool = False,
        mode: Union[str, ServiceLogMode] = ServiceLogMode.TAIL,
        max_lines: int = 1000,
    ) -> str:
        """Get the controller logs of a service."""
        return self._private_sdk.controller_logs(
            name=name,
            cloud=cloud,
            project=project,
            canary=canary,
            mode=mode,
            max_lines=max_lines,
        )

    @sdk_docs(doc_py_example=_TAGS_ADD_EXAMPLE, arg_docstrings=_TAGS_ADD_ARG_DOCSTRINGS)
    def add_tags(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        tags: Dict[str, str],
    ) -> None:
        """Upsert (add/update) tag key/value pairs for a service."""
        return self._private_sdk.add_tags(
            id=id, name=name, cloud=cloud, project=project, tags=tags
        )

    @sdk_docs(
        doc_py_example=_TAGS_REMOVE_EXAMPLE, arg_docstrings=_TAGS_REMOVE_ARG_DOCSTRINGS
    )
    def remove_tags(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        keys: List[str],
    ) -> None:
        """Remove tags by key from a service."""
        return self._private_sdk.remove_tags(
            id=id, name=name, cloud=cloud, project=project, keys=keys
        )

    @sdk_docs(
        doc_py_example=_TAGS_LIST_EXAMPLE, arg_docstrings=_TAGS_LIST_ARG_DOCSTRINGS
    )
    def list_tags(  # noqa: F811
        self,
        *,
        id: Optional[str] = None,  # noqa: A002
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> Dict[str, str]:
        """List tags for a service."""
        return self._private_sdk.list_tags(
            id=id, name=name, cloud=cloud, project=project
        )



from dataclasses import dataclass, field
from typing import ClassVar, Dict

from anyscale._private.models.model_base import ModelBase, ModelEnum


class IntegrationType(ModelEnum):
    """Type of third-party integration for connections."""

    DATABRICKS = "DATABRICKS"

    __docstrings__: ClassVar[Dict[str, str]] = {
        DATABRICKS: "Databricks integration for Unity Catalog access",
    }


class ConnectionType:
    """Backend connection type strings."""

    DATABRICKS_U2M = "databricks_U2M"


CONNECTION_TYPE_TO_INTEGRATION_TYPE: Dict[str, IntegrationType] = {
    ConnectionType.DATABRICKS_U2M: IntegrationType.DATABRICKS,  # type: ignore[dict-item]
}


@dataclass(frozen=True)
class ConnectionConfig(ModelBase):
    """Configuration for a third-party integration connection.

    Connections allow workloads (jobs, workspaces, etc.) to access external services
    like Databricks Unity Catalog. Each connection is identified by its integration
    type and name.
    """

    __doc_py_example__ = """\
from anyscale._private.models.integrations import ConnectionConfig, IntegrationType

connection = ConnectionConfig(
    integration_type=IntegrationType.DATABRICKS,
    connection_name="my-databricks-connection",
)
"""

    __doc_yaml_example__ = """\
connections:
  - integration_type: DATABRICKS
    connection_name: my-databricks-connection
"""

    integration_type: IntegrationType = field(
        metadata={"docstring": "The type of integration (e.g., DATABRICKS)."},
    )

    def _validate_integration_type(
        self, integration_type: IntegrationType
    ) -> IntegrationType:
        if not isinstance(integration_type, IntegrationType):
            raise TypeError(
                f"'integration_type' must be an 'IntegrationType' (it is {type(integration_type)})."
            )
        return integration_type

    connection_name: str = field(
        metadata={
            "docstring": "The name of the connection as registered in the organization settings.",
        },
    )

    def _validate_connection_name(self, connection_name: str):
        if not isinstance(connection_name, str):
            raise TypeError(
                f"'connection_name' must be a string (it is {type(connection_name)})."
            )
        if not connection_name:
            raise ValueError("'connection_name' cannot be empty.")



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class WorkloadStateInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'workload_name': 'str',
        'workload_type': 'str',
        'workload_start_time': 'datetime',
        'workload_cloud': 'str',
        'workload_project': 'str',
        'workload_creator': 'str',
        'url': 'str',
        'tags': 'list[TagKeyValue]',
        'workload_machine_info': 'list[WorkloadMachineInfo]'
    }

    attribute_map = {
        'workload_name': 'workload_name',
        'workload_type': 'workload_type',
        'workload_start_time': 'workload_start_time',
        'workload_cloud': 'workload_cloud',
        'workload_project': 'workload_project',
        'workload_creator': 'workload_creator',
        'url': 'url',
        'tags': 'tags',
        'workload_machine_info': 'workload_machine_info'
    }

    def __init__(self, workload_name=None, workload_type=None, workload_start_time=None, workload_cloud=None, workload_project='', workload_creator='', url='', tags=None, workload_machine_info=None, local_vars_configuration=None):  # noqa: E501
        """WorkloadStateInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._workload_name = None
        self._workload_type = None
        self._workload_start_time = None
        self._workload_cloud = None
        self._workload_project = None
        self._workload_creator = None
        self._url = None
        self._tags = None
        self._workload_machine_info = None
        self.discriminator = None

        self.workload_name = workload_name
        self.workload_type = workload_type
        self.workload_start_time = workload_start_time
        self.workload_cloud = workload_cloud
        if workload_project is not None:
            self.workload_project = workload_project
        if workload_creator is not None:
            self.workload_creator = workload_creator
        if url is not None:
            self.url = url
        if tags is not None:
            self.tags = tags
        self.workload_machine_info = workload_machine_info

    @property
    def workload_name(self):
        """Gets the workload_name of this WorkloadStateInfo.  # noqa: E501

        The name of the workload.  # noqa: E501

        :return: The workload_name of this WorkloadStateInfo.  # noqa: E501
        :rtype: str
        """
        return self._workload_name

    @workload_name.setter
    def workload_name(self, workload_name):
        """Sets the workload_name of this WorkloadStateInfo.

        The name of the workload.  # noqa: E501

        :param workload_name: The workload_name of this WorkloadStateInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and workload_name is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_name`, must not be `None`")  # noqa: E501

        self._workload_name = workload_name

    @property
    def workload_type(self):
        """Gets the workload_type of this WorkloadStateInfo.  # noqa: E501

        The type of the workload.  # noqa: E501

        :return: The workload_type of this WorkloadStateInfo.  # noqa: E501
        :rtype: str
        """
        return self._workload_type

    @workload_type.setter
    def workload_type(self, workload_type):
        """Sets the workload_type of this WorkloadStateInfo.

        The type of the workload.  # noqa: E501

        :param workload_type: The workload_type of this WorkloadStateInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and workload_type is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_type`, must not be `None`")  # noqa: E501

        self._workload_type = workload_type

    @property
    def workload_start_time(self):
        """Gets the workload_start_time of this WorkloadStateInfo.  # noqa: E501

        The start time of the workload.  # noqa: E501

        :return: The workload_start_time of this WorkloadStateInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._workload_start_time

    @workload_start_time.setter
    def workload_start_time(self, workload_start_time):
        """Sets the workload_start_time of this WorkloadStateInfo.

        The start time of the workload.  # noqa: E501

        :param workload_start_time: The workload_start_time of this WorkloadStateInfo.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and workload_start_time is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_start_time`, must not be `None`")  # noqa: E501

        self._workload_start_time = workload_start_time

    @property
    def workload_cloud(self):
        """Gets the workload_cloud of this WorkloadStateInfo.  # noqa: E501

        The cloud of the workload.  # noqa: E501

        :return: The workload_cloud of this WorkloadStateInfo.  # noqa: E501
        :rtype: str
        """
        return self._workload_cloud

    @workload_cloud.setter
    def workload_cloud(self, workload_cloud):
        """Sets the workload_cloud of this WorkloadStateInfo.

        The cloud of the workload.  # noqa: E501

        :param workload_cloud: The workload_cloud of this WorkloadStateInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and workload_cloud is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_cloud`, must not be `None`")  # noqa: E501

        self._workload_cloud = workload_cloud

    @property
    def workload_project(self):
        """Gets the workload_project of this WorkloadStateInfo.  # noqa: E501

        The project of the workload.  # noqa: E501

        :return: The workload_project of this WorkloadStateInfo.  # noqa: E501
        :rtype: str
        """
        return self._workload_project

    @workload_project.setter
    def workload_project(self, workload_project):
        """Sets the workload_project of this WorkloadStateInfo.

        The project of the workload.  # noqa: E501

        :param workload_project: The workload_project of this WorkloadStateInfo.  # noqa: E501
        :type: str
        """

        self._workload_project = workload_project

    @property
    def workload_creator(self):
        """Gets the workload_creator of this WorkloadStateInfo.  # noqa: E501

        The creator of the workload.  # noqa: E501

        :return: The workload_creator of this WorkloadStateInfo.  # noqa: E501
        :rtype: str
        """
        return self._workload_creator

    @workload_creator.setter
    def workload_creator(self, workload_creator):
        """Sets the workload_creator of this WorkloadStateInfo.

        The creator of the workload.  # noqa: E501

        :param workload_creator: The workload_creator of this WorkloadStateInfo.  # noqa: E501
        :type: str
        """

        self._workload_creator = workload_creator

    @property
    def url(self):
        """Gets the url of this WorkloadStateInfo.  # noqa: E501

        Relative URL for deeplink to workload detail page.  # noqa: E501

        :return: The url of this WorkloadStateInfo.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this WorkloadStateInfo.

        Relative URL for deeplink to workload detail page.  # noqa: E501

        :param url: The url of this WorkloadStateInfo.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def tags(self):
        """Gets the tags of this WorkloadStateInfo.  # noqa: E501

        User Managed tags associated with this workload.  # noqa: E501

        :return: The tags of this WorkloadStateInfo.  # noqa: E501
        :rtype: list[TagKeyValue]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this WorkloadStateInfo.

        User Managed tags associated with this workload.  # noqa: E501

        :param tags: The tags of this WorkloadStateInfo.  # noqa: E501
        :type: list[TagKeyValue]
        """

        self._tags = tags

    @property
    def workload_machine_info(self):
        """Gets the workload_machine_info of this WorkloadStateInfo.  # noqa: E501

        The machines allocated to this workload.  # noqa: E501

        :return: The workload_machine_info of this WorkloadStateInfo.  # noqa: E501
        :rtype: list[WorkloadMachineInfo]
        """
        return self._workload_machine_info

    @workload_machine_info.setter
    def workload_machine_info(self, workload_machine_info):
        """Sets the workload_machine_info of this WorkloadStateInfo.

        The machines allocated to this workload.  # noqa: E501

        :param workload_machine_info: The workload_machine_info of this WorkloadStateInfo.  # noqa: E501
        :type: list[WorkloadMachineInfo]
        """
        if self.local_vars_configuration.client_side_validation and workload_machine_info is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_machine_info`, must not be `None`")  # noqa: E501

        self._workload_machine_info = workload_machine_info

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkloadStateInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, WorkloadStateInfo):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class Dataset(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'filename': 'str',
        'description': 'str',
        'name': 'str',
        'project_id': 'str',
        'id': 'str',
        'creator_id': 'str',
        'created_at': 'datetime',
        'deleted_at': 'datetime',
        'cloud_id': 'str',
        'num_versions': 'int',
        'storage_uri': 'str',
        'version': 'int'
    }

    attribute_map = {
        'filename': 'filename',
        'description': 'description',
        'name': 'name',
        'project_id': 'project_id',
        'id': 'id',
        'creator_id': 'creator_id',
        'created_at': 'created_at',
        'deleted_at': 'deleted_at',
        'cloud_id': 'cloud_id',
        'num_versions': 'num_versions',
        'storage_uri': 'storage_uri',
        'version': 'version'
    }

    def __init__(self, filename=None, description=None, name=None, project_id=None, id=None, creator_id=None, created_at=None, deleted_at=None, cloud_id=None, num_versions=None, storage_uri=None, version=None, local_vars_configuration=None):  # noqa: E501
        """Dataset - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._filename = None
        self._description = None
        self._name = None
        self._project_id = None
        self._id = None
        self._creator_id = None
        self._created_at = None
        self._deleted_at = None
        self._cloud_id = None
        self._num_versions = None
        self._storage_uri = None
        self._version = None
        self.discriminator = None

        self.filename = filename
        if description is not None:
            self.description = description
        self.name = name
        self.project_id = project_id
        self.id = id
        self.creator_id = creator_id
        self.created_at = created_at
        if deleted_at is not None:
            self.deleted_at = deleted_at
        self.cloud_id = cloud_id
        self.num_versions = num_versions
        self.storage_uri = storage_uri
        self.version = version

    @property
    def filename(self):
        """Gets the filename of this Dataset.  # noqa: E501


        :return: The filename of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._filename

    @filename.setter
    def filename(self, filename):
        """Sets the filename of this Dataset.


        :param filename: The filename of this Dataset.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and filename is None:  # noqa: E501
            raise ValueError("Invalid value for `filename`, must not be `None`")  # noqa: E501

        self._filename = filename

    @property
    def description(self):
        """Gets the description of this Dataset.  # noqa: E501


        :return: The description of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Dataset.


        :param description: The description of this Dataset.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def name(self):
        """Gets the name of this Dataset.  # noqa: E501


        :return: The name of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Dataset.


        :param name: The name of this Dataset.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def project_id(self):
        """Gets the project_id of this Dataset.  # noqa: E501


        :return: The project_id of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this Dataset.


        :param project_id: The project_id of this Dataset.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and project_id is None:  # noqa: E501
            raise ValueError("Invalid value for `project_id`, must not be `None`")  # noqa: E501

        self._project_id = project_id

    @property
    def id(self):
        """Gets the id of this Dataset.  # noqa: E501


        :return: The id of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Dataset.


        :param id: The id of this Dataset.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def creator_id(self):
        """Gets the creator_id of this Dataset.  # noqa: E501


        :return: The creator_id of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._creator_id

    @creator_id.setter
    def creator_id(self, creator_id):
        """Sets the creator_id of this Dataset.


        :param creator_id: The creator_id of this Dataset.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and creator_id is None:  # noqa: E501
            raise ValueError("Invalid value for `creator_id`, must not be `None`")  # noqa: E501

        self._creator_id = creator_id

    @property
    def created_at(self):
        """Gets the created_at of this Dataset.  # noqa: E501


        :return: The created_at of this Dataset.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Dataset.


        :param created_at: The created_at of this Dataset.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def deleted_at(self):
        """Gets the deleted_at of this Dataset.  # noqa: E501


        :return: The deleted_at of this Dataset.  # noqa: E501
        :rtype: datetime
        """
        return self._deleted_at

    @deleted_at.setter
    def deleted_at(self, deleted_at):
        """Sets the deleted_at of this Dataset.


        :param deleted_at: The deleted_at of this Dataset.  # noqa: E501
        :type: datetime
        """

        self._deleted_at = deleted_at

    @property
    def cloud_id(self):
        """Gets the cloud_id of this Dataset.  # noqa: E501


        :return: The cloud_id of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this Dataset.


        :param cloud_id: The cloud_id of this Dataset.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def num_versions(self):
        """Gets the num_versions of this Dataset.  # noqa: E501


        :return: The num_versions of this Dataset.  # noqa: E501
        :rtype: int
        """
        return self._num_versions

    @num_versions.setter
    def num_versions(self, num_versions):
        """Sets the num_versions of this Dataset.


        :param num_versions: The num_versions of this Dataset.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and num_versions is None:  # noqa: E501
            raise ValueError("Invalid value for `num_versions`, must not be `None`")  # noqa: E501

        self._num_versions = num_versions

    @property
    def storage_uri(self):
        """Gets the storage_uri of this Dataset.  # noqa: E501


        :return: The storage_uri of this Dataset.  # noqa: E501
        :rtype: str
        """
        return self._storage_uri

    @storage_uri.setter
    def storage_uri(self, storage_uri):
        """Sets the storage_uri of this Dataset.


        :param storage_uri: The storage_uri of this Dataset.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and storage_uri is None:  # noqa: E501
            raise ValueError("Invalid value for `storage_uri`, must not be `None`")  # noqa: E501

        self._storage_uri = storage_uri

    @property
    def version(self):
        """Gets the version of this Dataset.  # noqa: E501


        :return: The version of this Dataset.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this Dataset.


        :param version: The version of this Dataset.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and version is None:  # noqa: E501
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Dataset):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Dataset):
            return True

        return self.to_dict() != other.to_dict()
